package com.cim.idm.framework;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.sql.DataSource;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.env.Environment;
import org.springframework.jdbc.support.lob.LobHandler;
import org.springframework.stereotype.Service;

import com.cim.idm.flowpackage.core.BpelProcessManager;
import com.cim.idm.flowpackage.core.activity.Activity;
import com.cim.idm.flowpackage.core.activity.BpelProcess;
import com.cim.idm.framework.action.management.ActionContainerManager;
import com.cim.idm.framework.esb.GenericSender;
import com.cim.idm.framework.esb.SenderProxy;
import com.cim.idm.framework.esb.email.EmailTransport;
import com.cim.idm.framework.event.MessageEventAdaptor;
import com.cim.idm.framework.exception.ErrorSignal;
import com.cim.idm.framework.exception.IDMFrameErrorSignal;
import com.cim.idm.framework.exception.ext.ExceptionMessages;
import com.cim.idm.framework.orm.ObjectAttributeMap;
import com.cim.idm.framework.orm.OrmStandardEngine;
import com.cim.idm.framework.orm.SqlTemplate;
import com.cim.idm.framework.shutdown.ShutdownableShutdowner;
import com.cim.idm.framework.transaction.IDMFrameTransactionManager;
import com.cim.idm.framework.transaction.TransactionManager;
import com.cim.idm.framework.transaction.TxDataSourceManager;
import com.cim.idm.framework.util.service.IDMFrameProperties;
import com.cim.idm.framework.util.service.ServiceUtil;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework
 *  NAME    : IDMFrameServiceProxy.java
 *  TYPE    : JAVA
 *  DESCRIPTION :
 *
 ****************************************************************************
 */

@Service
@DependsOn(value = {"bpelProcessManager"})
public class IDMFrameServiceProxy implements ApplicationContextAware
{
	private static Log					log		= LogFactory.getLog( IDMFrameServiceProxy.class );

	private static Map<Long, Thread>	threadMap	= new ConcurrentHashMap<Long, Thread>();
	
	private static int bpelCompletionTimeout		= 5000;
	private static int shutdownableTimeout			= 5000;
	private static int bidmbootCloseTimeout				= 5000;

	@Autowired
	private static ApplicationContext applicationContext = null;
//	private static boolean flowOptionOn				= false;
	
	private static boolean shutdownOnAppContextError = false;
	
	@Autowired
	Environment environment;
	/**
	 * Start of BIDM Server
	 */

	private static void assertContextInjected() {
		Validate.validState(applicationContext != null, "applicaitonContext属性未注入,.");
	}
	@PostConstruct
	public void start() throws Exception
	{
		this.readSystemProperties();
		this.addSystemProperties();
	}
	
	/**
	 * Stop of BIDM Server
	 */
	@PreDestroy
	public void stop() throws Exception
	{
		log.info("BIDM Server shutdown..");
		IDMFrameServiceProxy.closeListeners();
		//@DependsOn(value = {"bpelProcessManager"}) will destroy bpelProcessManager after destroy this bean , so do not need execute IDMFrameServiceProxy.shutdownBpelProcessManager();
//		IDMFrameServiceProxy.shutdownBpelProcessManager();
		IDMFrameServiceProxy.shutdownShutdownables();
		applicationContext = null;
	}

	/**
	 * set bundleContext.
	 * 
	 * @param bundleContext
	 */
	public void setApplicationContext(ApplicationContext applicationContext)
	{
		IDMFrameServiceProxy.applicationContext = applicationContext;

		if (!threadMap.isEmpty())
		{
			Iterator<Thread> iter = threadMap.values().iterator();
			while (iter.hasNext())
			{
				iter.next().interrupt();
			}
		}
	}

	public static ApplicationContext getApplicationContext()
	{
		assertContextInjected();
		if (applicationContext == null)
		{
			Thread thread = Thread.currentThread();
			log.info("GET CurrentThread : " + thread.getName());
			threadMap.put(thread.getId(), thread);

			try
			{
				Thread.sleep(ServiceUtil.getServiceLookupTimeout());
			} 
			catch (InterruptedException ex)
			{
				return applicationContext;
			} 
			finally
			{
				threadMap.remove(thread.getId());
			}

			throw new IDMFrameErrorSignal(ErrorSignal.NotActive, "idmframe.kernel is not Active.");
		}
		else
			return applicationContext;
	}

	@Deprecated
	public static void terminateImmediately()
	{
		log.info("***************************************************************************************");
		System.exit(0);
	}

	@Deprecated
	public static void terminateAfterManagement()
	{
		log.info("***************************************************************************************");
		log.info("Closing message listeners .....");
		MessageEventAdaptor MessageEventAdaptor = (MessageEventAdaptor) applicationContext.getBean(MessageEventAdaptor.class.getName());
		MessageEventAdaptor.terminate();

		try
		{
			Thread.sleep( 100 );
		}
		catch ( Exception ex ){}
		
		BpelProcessManager bpelProcessManager =	(BpelProcessManager) applicationContext.getBean(BpelProcessManager.class.getName());
		
		if ( bpelProcessManager != null )
		{
			log.info("Checking running bpels");
			
			int waittingCount = bpelCompletionTimeout / 1000;
			int i = 0;
			
			for (; i < waittingCount; i++)
			{
				if ( bpelProcessManager.getRunningBpelProcessSize() == 0 )
				{
					log.info( "No running bpels" );
					break;
				}
				else
				{
					log.info( "Waitting for running bpels(" + bpelProcessManager.getRunningBpelProcessSize() + ") to be completed" );
					
					try
					{
						Thread.sleep( 1000 );
					}
					catch ( Exception ex ){}
				}
			}
			
			if ( i == waittingCount )
			{
				log.warn( "Bpel Completion Waitting Timeout. Will be terminated." );
				
				Map<String, Activity> map = bpelProcessManager.getRunningBpelProcesses();
				
				Iterator<Entry<String, Activity>> iter = map.entrySet().iterator();
				while ( iter.hasNext())
				{
					Entry<String, Activity> entry = iter.next();
					Activity activity = entry.getValue();
					long duration = System.currentTimeMillis() - activity.getStartTime().getTime();
					
					String runningBpel = activity.getName() + " [Activity=" + ((BpelProcess)activity).getRunningActivity() + ", Duration=" + duration + "]";
					log.info( "Running Bpel : " + runningBpel );
				}
			}
			
			bpelProcessManager.getThreadPool().shutdownNow();
		}
		
		log.info("***************************************************************************************");
		
	}

	public static TxDataSourceManager getTxDataSourceManager()
	{
		return (TxDataSourceManager) applicationContext.getBean(TxDataSourceManager.class);
	}

	public static IDMFrameTransactionManager getTransactionManager()
	{
		return ( IDMFrameTransactionManager ) applicationContext.getBean( IDMFrameTransactionManager.class);
	}
	
	public static TransactionManager getTransactionManager( Class<? extends TransactionManager> cls )
	{
		return (TransactionManager) applicationContext.getBean( cls);
	}

	@SuppressWarnings("unchecked")
	public static GenericSender<String,String> getGenericSender()
	{
		return (GenericSender<String,String>) applicationContext.getBean(GenericSender.class);
	}
	
	public static SenderProxy getSenderProxy()
	{
		return (SenderProxy) applicationContext.getBean( SenderProxy.class);
	}

	public static EmailTransport getEmailTransport()
	{
		return (EmailTransport) applicationContext.getBean(EmailTransport.class);
	}
/*
	@Deprecated
	public static CacheManager getCacheManager()
	{
		return (CacheManager) BundleUtil.waitForService(CacheManager.class.getName());
	}
*/
//	@Deprecated
//	public static OracleLobHandler getOracleLobHandler()
//	{
//		return (OracleLobHandler) BundleUtil.waitForService(OracleLobHandler.class.getName());
//	}

	public static LobHandler getLobHandler()
	{
		return (LobHandler) applicationContext.getBean(LobHandler.class);
	}

	public static DataSource getDataSource()
	{
		return (DataSource) applicationContext.getBean(DataSource.class);
	}

	public static SqlTemplate getSqlTemplate()
	{
		return (SqlTemplate) applicationContext.getBean(SqlTemplate.class);
	}

	public static ExceptionMessages getExceptionMessages()
	{
		return (ExceptionMessages) applicationContext.getBean(ExceptionMessages.class);
	}
	
	
	@SuppressWarnings( "rawtypes" )
	public static OrmStandardEngine getOrmStandardEngine()
	{
		return (OrmStandardEngine) applicationContext.getBean(OrmStandardEngine.class);
	}

	public static ObjectAttributeMap getObjectAttributeMap()
	{
		return (ObjectAttributeMap) applicationContext.getBean(ObjectAttributeMap.class);
	}

	/**
	 * read System Properties
	 */
	private void readSystemProperties()
	{
		String timeout = null;
		
		timeout = environment.getProperty( IDMFrameProperties.BPEL_COMPLETION_TIMEOUT );
		if ( StringUtils.isNotEmpty( timeout ) && StringUtils.isNumeric( timeout ))
			bpelCompletionTimeout = Integer.parseInt( timeout );
		
		timeout = environment.getProperty( IDMFrameProperties.SHUTDOWNABLE_TIMEOUT );
		if ( StringUtils.isNotEmpty( timeout ) && StringUtils.isNumeric( timeout ))
			shutdownableTimeout = Integer.parseInt( timeout );
		
		timeout = environment.getProperty( IDMFrameProperties.BIDMBOOT_CLOSE_TIMEOUT );
		if ( StringUtils.isNotEmpty( timeout ) && StringUtils.isNumeric( timeout ))
			bidmbootCloseTimeout = Integer.parseInt( timeout );
		
		String shutdown = environment.getProperty( IDMFrameProperties.SHUTDOWN_APPCONTEXT_ERR );
		if ( "true".equalsIgnoreCase( shutdown ) || "false".equalsIgnoreCase( shutdown ))
			shutdownOnAppContextError = Boolean.parseBoolean( shutdown );
		
//		String boostOption = System.getProperty(IDMFrameProperties.FLOW_OPTION);
//		if ("idmFlow".equalsIgnoreCase(boostOption))
//			setFlowOptionOn(true);
	}
	
	/**
	 * System property is set.
	 */
	private void addSystemProperties()
	{
		try
		{
			System.setProperty( "idmframe.ip", InetAddress.getLocalHost().getHostAddress());
			System.setProperty( "idmframe.hostname", InetAddress.getLocalHost().getHostName());
		}
		catch ( UnknownHostException ex )
		{}
	}
	
	/**
	 * close message listeners
	 */
	public static void closeListeners()
	{
		MessageEventAdaptor messageEventAdaptor = null;
		Object obj = null;
		try
		{
			obj = applicationContext.getBean(MessageEventAdaptor.class);
		}
		catch ( Exception ex )
		{
			log.info( "No MessageEventAdaptor" );
		}
		
		if(obj != null) {
			messageEventAdaptor = (MessageEventAdaptor) obj;
			log.info("Closing message listeners");
			try {
				messageEventAdaptor.terminate();
			}catch(Exception ex) {
				log.error(ex, ex);
			}
		}
	}
	
	/**
	 * close BpelProcessManger
	 */
	public static void shutdownBpelProcessManager()
	{
		try
		{
			BpelProcessManager bpelProcessManager = (BpelProcessManager)applicationContext.getBean(BpelProcessManager.class);
	
			if ( bpelProcessManager == null )
					
				log.info( "No BpelProcessManager" );
			else
			{
				log.info( "Closing BpelProcessManger(" + IDMFrameProperties.BPEL_COMPLETION_TIMEOUT + "=" + bpelCompletionTimeout + ")" );
				bpelProcessManager.shutdown( bpelCompletionTimeout );
			}

		}
		catch ( NoClassDefFoundError ex )
		{
			log.info( "No BpelProcessManager" );
		}
		catch ( Exception ex )
		{
			log.error( ex, ex );
		}
	}
	
	
	
	/**
	 * close ShutdownableShutdowner
	 */
	public static void shutdownShutdownables()
	{
		log.info( "Closing Shutdownables(" + IDMFrameProperties.SHUTDOWNABLE_TIMEOUT + "=" + shutdownableTimeout + ")" );
		
		try
		{
			Thread th = new Thread( new ShutdownableShutdowner( shutdownableTimeout ).setApplicationContext(applicationContext));
			th.start();
			
			if ( th.isAlive())
				th.join( shutdownableTimeout );
			
			if ( th.isAlive())
				log.info( "Shutdownable Completion Waitting Timeout." );
		}
		catch ( Exception ex )
		{
			log.error( ex, ex );
		}
	}
	
	public static ActionContainerManager getActionContainerManager()
	{
		return applicationContext.getBean(ActionContainerManager.class);
	}

}
