package com.cim.idm.framework.action.management;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import xutil4j.xml.DomUtils;

import com.cim.idm.framework.action.info.ActionFormat;
import com.cim.idm.framework.action.info.ModelingKey;
import com.cim.idm.framework.action.info.ValidationFormat;
import com.cim.idm.framework.exception.ext.FrameworkErrorSignal;


public class ActionContainer {

	private static Log	logger		= LogFactory.getLog( ActionContainer.class );
	
	private String actionName = "";
	
	private boolean defaultAction = false;
	
	private HashMap<String, List<ActionFormat>> actionMap = new HashMap<String, List<ActionFormat>>();
	private HashMap<String, HashMap<String, ValidationFormat>> validationMap = new HashMap<String, HashMap<String, ValidationFormat>>();
	
	private HashMap<String, Object> beanFActory = new HashMap<String, Object>();
	
	public void initialize(String actionName, String fileName) throws TransformerException
	{
		this.actionName = actionName;
		//load xmlFile and fills actionMap , key is type.operationName
		Document doc = null;
		try {
			doc = DomUtils.load(fileName);
		} catch (ParserConfigurationException e) {
			logger.error("["+actionName+"]File Name=" + fileName + " Error:" +  e.getMessage());
			return;
		} catch (SAXException e) {
			logger.error("["+actionName+"]File Name=" + fileName + " Error:" +  e.getMessage());
			return;
		} catch (IOException e) {
			logger.error("["+actionName+"]File Name=" + fileName + " Error:" +  e.getMessage());
			return;
		}
		
		fillsActionValidationMap(doc);
		
	}
	
	private boolean fillsActionValidationMap(Document doc)
	{
		String StartTime = new SimpleDateFormat("HH:mm:ss SSS").format(new Date());
		
		if(doc.getDocumentElement().getNodeName() != ModelingKey.ROOTNAME)
		{
			logger.warn("["+actionName+"] Wrong XML ROOT");
			return false;
		}
		
		String defaultAction = doc.getDocumentElement().getAttributes().getNamedItem(ModelingKey.DEFAULT).getTextContent();
		try{
			this.defaultAction = Boolean.parseBoolean(defaultAction);
		}catch(Exception e){}
		
		for(int i=0; i < doc.getDocumentElement().getChildNodes().getLength();i++)
    	{
			Node subItem = (Node) doc.getDocumentElement().getChildNodes().item(i);
    		
    		if(subItem.getNodeType() != Node.ELEMENT_NODE) continue;
    		
    		String moduleName = subItem.getAttributes().getNamedItem(ModelingKey.NAME).getTextContent();
    		
    		String moduleString = DomUtils.toString(subItem);
    		//String firstChild = DomUtils.toString(subItem.getFirstChild());
    		NodeList operations = subItem.getChildNodes(); 
    		for(int j=0; j< operations.getLength(); j++)
    		{
    			Node operationNode = operations.item(j);
    			if(operationNode.getNodeType() != Node.ELEMENT_NODE) continue;
    			try {
					parseOperation(moduleName, operationNode);
				} catch (FrameworkErrorSignal e) {
					// TODO Auto-generated catch block
					logger.error(e);
					return false;
				}
    		}
    	}		
		
		return true;
	}
		
	private void parseOperation(String moduleName, Node operationNode) throws FrameworkErrorSignal
	{
		String nodeString = DomUtils.toString(operationNode);
		
		String operationName = operationNode.getAttributes().getNamedItem(ModelingKey.NAME).getTextContent();
		
		if(operationName == null || operationName.equals(""))
		{
			logger.warn("["+actionName+"] operation can't be empty");
			return;
		}

		List<ActionFormat> actionlist = new ArrayList<ActionFormat>();
		HashMap<String,ValidationFormat> validationlist = new HashMap<String, ValidationFormat>();

		//get actions
		NodeList actionNvalidation = operationNode.getChildNodes();
		for(int n=0; n< actionNvalidation.getLength(); n++)
		{
			Node item = actionNvalidation.item(n);
			if(item.getNodeType() != Node.ELEMENT_NODE) continue;
			String itemName = item.getNodeName();
			if(itemName.equals(ModelingKey.ACTIONS))
			{
				NodeList actions = item.getChildNodes();
				for(int i=0; i< actions.getLength(); i++)
				{
					Node action = actions.item(i);
					if(action.getNodeType() != Node.ELEMENT_NODE) continue;
					ActionFormat actionformat = new ActionFormat();
					actionformat.setDoAction(Boolean.parseBoolean(action.getAttributes().getNamedItem(ModelingKey.USE).getTextContent()));
					String actionClass = action.getAttributes().getNamedItem(ModelingKey.CLASS).getTextContent();
					actionformat.setClassName(actionClass);
					actionformat.setActionName(action.getTextContent());
					actionlist.add(actionformat);			
				}
				if(actionMap.get(moduleName + "::" + operationName)!=null){
					throw new RuntimeException(moduleName + "::" + operationName + " has already exist! ");
				}
				actionMap.put(moduleName + "::" + operationName , actionlist);
			}
			else if(itemName.equals(ModelingKey.VALIDATIONS))
			{
				NodeList validations = item.getChildNodes();
				for(int i=0; i< validations.getLength(); i++)
				{
					Node validation = validations.item(i);
					if(validation.getNodeType() != Node.ELEMENT_NODE) continue;
					ValidationFormat validationformat = new ValidationFormat();
					validationformat.setDoValidate(Boolean.parseBoolean(validation.getAttributes().getNamedItem(ModelingKey.USE).getTextContent()));
					//validationformat.setClassName(validation.getAttributes().getNamedItem(ModelingKey.CLASS).getTextContent());
					validationformat.setValidationName(validation.getTextContent());
					validationlist.put(validationformat.getValidationName(),validationformat);			
				}
				validationMap.put(moduleName + "::" + operationName , validationlist);
			}
		}		
	}
	
	public List<ActionFormat> getActionFormats(String moduleName, String operationName)
	{
		return actionMap.get(moduleName + "::" + operationName);
	}
	
	public HashMap<String, ValidationFormat> getValidationFormats(String moduleName, String operationName)
	{
		return validationMap.get(moduleName + "::" + operationName);
	}
		
	
	public boolean isDefaultAction() {
		return defaultAction;
	}

	public void setDefaultAction(boolean defaultAction) {
		this.defaultAction = defaultAction;
	}

	public HashMap<String, List<ActionFormat>> getActionMap() {
		return actionMap;
	}
	
}
