package com.cim.idm.framework.action.management;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;

import javax.xml.transform.TransformerException;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.cim.idm.framework.action.info.ActionFormat;
import com.cim.idm.framework.action.info.ActionFormatAdditionalInfo;
import com.cim.idm.framework.action.info.ValidationFormat;
import com.cim.idm.framework.data.EventInfo;
import com.cim.idm.framework.data.TransitionInfo;
import com.cim.idm.framework.exception.ext.DuplicateNameSignal;
import com.cim.idm.framework.exception.ext.ExceptionKey;
import com.cim.idm.framework.exception.ext.FrameworkErrorSignal;
import com.cim.idm.framework.exception.ext.InvalidStateTransitionSignal;
import com.cim.idm.framework.exception.ext.LocalizedException;
import com.cim.idm.framework.exception.ext.NotFoundSignal;
import com.cim.idm.framework.orm.PersistentClass;
import com.cim.idm.framework.orm.info.DataInfo;
import com.cim.idm.framework.orm.info.KeyInfo;

public class ActionContainerManager<KEY extends KeyInfo, DATA extends DataInfo, INFO extends TransitionInfo> implements ApplicationContextAware {
	private static Log logger = LogFactory.getLog(ActionContainerManager.class);

	private HashMap<String, Class> classFactory = new HashMap<String, Class>();

	private ApplicationContext ac;

	// private List<String> bundleSymbolicNameList;

	private HashMap<String, String> actionFileList = new HashMap<String, String>();

	private HashMap<String, ActionContainer> actionContainers = new HashMap<String, ActionContainer>();

	private ActionContainer defaultActionContainer = null;

	private String configRootPath;

	public String getConfigRootPath() {
		return configRootPath;
	}

	public void setConfigRootPath(String configRootPath) {
		this.configRootPath = configRootPath;
	}

	public void setApplicationContext(ApplicationContext arg0) throws BeansException {
		this.ac = arg0;
	}

	public void initialize() {
		if(!StringUtils.isEmpty(configRootPath) && !configRootPath.endsWith("/"))
			configRootPath = configRootPath + "/";
		//load actionFileList
		for (String key : getActionFileList().keySet()) {
			String fileName = getActionFileList().get(key);
//			Bundle bundle = null;

			ActionContainer actionContainer = new ActionContainer();
			try {
//				bundle = BundleUtil.getBundle( bc.getBundles(), bundleSymbolicNameList.get(0));
//				File actionfile = BundleUtil.getBundleResource( bundle, "/META-INF/actions/" + fileName );
				File actionfile = new File(configRootPath + fileName);

				actionContainer.initialize(key, actionfile.getPath());
				if (actionContainer.isDefaultAction())
					defaultActionContainer = actionContainer;
				actionContainers.put(key, actionContainer);
			} catch (TransformerException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * 
	 * @param actionType    in case of null, action_default. ex)action_default,
	 *                      action_NonHistory, action_ExceptAction, action_Glass
	 * @param moduleName
	 * @param operationName
	 */
	public DATA Execute(String actionTypeName, String moduleName, String operationName, DATA oldData,
			DATA newData, EventInfo eventInfo, INFO transitionInfo)
			throws NotFoundSignal, DuplicateNameSignal, InvalidStateTransitionSignal, FrameworkErrorSignal {
		Execute(getActionType(actionTypeName), moduleName, operationName, oldData, newData, eventInfo, transitionInfo);
		return newData;
	}

	/**
	 * 
	 * @param actionType    in case of null, action_default. ex)action_default,
	 *                      action_NonHistory, action_ExceptAction, action_Glass
	 * @param moduleName
	 * @param operationName
	 */
	public DATA[] ExecuteForBatch(String actionTypeName, String moduleName, String operationName,
			DATA[] oldDataList, DATA[] newDataList, EventInfo eventInfo, INFO[] transitionInfoArray)
			throws NotFoundSignal, DuplicateNameSignal, InvalidStateTransitionSignal, FrameworkErrorSignal {
		ExecuteForBatch(getActionType(actionTypeName), moduleName, operationName, oldDataList, newDataList, eventInfo,
				transitionInfoArray);
		return newDataList;
	}

	public PersistentClass ExecuteUpdateAtOnce(String actionTypeName, String moduleName, String operationName,
			DATA[] oldDataList, EventInfo eventInfo, INFO transitionInfo, PersistentClass persistentClass)
			throws NotFoundSignal, DuplicateNameSignal, InvalidStateTransitionSignal, FrameworkErrorSignal {
		ExecuteUpdateAtOnce(getActionType(actionTypeName), moduleName, operationName, oldDataList, eventInfo,
				transitionInfo, persistentClass);
		return persistentClass;
	}

	/**
	 * 
	 * @param actionType    in case of null, action_default. ex)action_default,
	 *                      action_NonHistory, action_ExceptAction, action_Glass
	 * @param moduleName
	 * @param operationName
	 */
	private void Execute(ActionTypeEnum actionType, String moduleName, String operationName, DATA oldData,
			DATA newData, EventInfo eventInfo, INFO transitionInfo)
			throws NotFoundSignal, DuplicateNameSignal, InvalidStateTransitionSignal, FrameworkErrorSignal {
//		if(actionType == null)
//			actionType = actionType.action_default;

		ActionContainer container = actionContainers.get(actionType.toString());

		List<ActionFormat> actionList = container.getActionFormats(moduleName, operationName);

		HashMap<String, ValidationFormat> validationList = container.getValidationFormats(moduleName, operationName);

		// if null, get default container values.
		if (actionList == null || actionList.size() == 0) {
			logger.info(String.format(
					"Get DefaultActionContainer's Actions For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			actionList = defaultActionContainer.getActionFormats(moduleName, operationName);

			if (actionList == null || actionList.size() == 0)
				logger.warn(String.format("[Warn] Action is null ( ActionType[%s], moduleName[%s], operationName[%s]",
						actionType, moduleName, operationName));

		}
		if (validationList == null || validationList.size() == 0) {
			logger.info(String.format(
					"Get DefaultActionContainer's Validations For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			validationList = defaultActionContainer.getValidationFormats(moduleName, operationName);

			if (validationList == null || validationList.size() == 0)
				logger.warn(String.format(
						"[Warn] validationList is null ( ActionType[%s], moduleName[%s], operationName[%s]", actionType,
						moduleName, operationName));
		}

		for (ActionFormat action : actionList) {
			if (action.isDoAction()) {
				// get class
				Class actionClass = classFactory.get(action.getClassName());
				if (actionClass == null) {
					try {
						actionClass = Class.forName(action.getClassName());
						classFactory.put(action.getClassName(), actionClass);
					} catch (ClassNotFoundException ce) {
						logger.warn(String.format("Fail to find Class for Action (moduleName[%s],ActionName[%s]",
								moduleName, action.getActionName()));
						continue;
					}
				}
				// get bundle
				Object beanInstance = ac.getBean(actionClass);
				try {

					Method method = actionClass.getMethod(action.getActionName(), oldData.getClass(), newData.getClass(),
							EventInfo.class, transitionInfo.getClass(), HashMap.class);
					method.invoke(beanInstance, oldData, newData, eventInfo, transitionInfo, validationList);
				} catch (IllegalArgumentException e) {
					throw new FrameworkErrorSignal("IllegalArgumentException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
							+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
				} catch (IllegalAccessException e) {
					throw new FrameworkErrorSignal("IllegalAccessException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
							+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
				} catch (InvocationTargetException e) {
					Throwable throwable = e.getTargetException();
					String errorCode = "";
					String errorMessage = "";
					if (throwable instanceof LocalizedException) {
						LocalizedException localException = (LocalizedException) throwable;
						errorCode = localException.getErrorCode();
						errorMessage = localException.getLocalizedMessage();
						errorMessage = String.format(
								"%s: Can not execute method '" + actionClass.getName() + "." + action.getActionName()
										+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'." + "Reason : %s",
								errorCode, errorMessage);
						if (throwable instanceof FrameworkErrorSignal)
							throw new FrameworkErrorSignal(errorCode, errorMessage, throwable);
						if (throwable instanceof InvalidStateTransitionSignal)
							throw new InvalidStateTransitionSignal(errorCode, errorMessage, throwable);
						else if (throwable instanceof DuplicateNameSignal)
							throw new DuplicateNameSignal(errorCode, errorMessage, throwable);
						else if (throwable instanceof NotFoundSignal)
							throw new NotFoundSignal(errorCode, errorMessage, throwable);
					}
					throw new FrameworkErrorSignal(e.getClass().getSimpleName(), e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
							+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
				} catch (NoSuchMethodException e) {
					try {
						Method method = actionClass.getMethod(action.getActionName(), DataInfo.class, DataInfo.class,
								EventInfo.class, TransitionInfo.class, HashMap.class);
						method.invoke(beanInstance, oldData, newData, eventInfo, transitionInfo, validationList);
					}catch (IllegalArgumentException e1) {
						throw new FrameworkErrorSignal("IllegalArgumentException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
					} catch (IllegalAccessException e1) {
						throw new FrameworkErrorSignal("IllegalAccessException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
					} catch (InvocationTargetException e1) {
						Throwable throwable = e1.getTargetException();
						String errorCode = "";
						String errorMessage = "";
						if (throwable instanceof LocalizedException) {
							LocalizedException localException = (LocalizedException) throwable;
							errorCode = localException.getErrorCode();
							errorMessage = localException.getLocalizedMessage();
							errorMessage = String.format(
									"%s: Can not execute method '" + actionClass.getName() + "." + action.getActionName()
											+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'." + "Reason : %s",
									errorCode, errorMessage);
							if (throwable instanceof FrameworkErrorSignal)
								throw new FrameworkErrorSignal(errorCode, errorMessage, throwable);
							if (throwable instanceof InvalidStateTransitionSignal)
								throw new InvalidStateTransitionSignal(errorCode, errorMessage, throwable);
							else if (throwable instanceof DuplicateNameSignal)
								throw new DuplicateNameSignal(errorCode, errorMessage, throwable);
							else if (throwable instanceof NotFoundSignal)
								throw new NotFoundSignal(errorCode, errorMessage, throwable);
						}
						throw new FrameworkErrorSignal(e1.getClass().getSimpleName(), e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
					} catch (NoSuchMethodException e1) {
						throw new FrameworkErrorSignal(ExceptionKey.UnExpected_Exception, e1,
								"Can not found method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldData.getClass().getSimpleName()+","+ newData.getClass().getSimpleName()+", EventInfo,"+transitionInfo.getClass()+", HashMap)'.");
					}
				}
			}
		}

	}

	/**
	 * 
	 * @param actionType    in case of null, action_default. ex)action_default,
	 *                      action_NonHistory, action_ExceptAction, action_Glass
	 * @param moduleName
	 * @param operationName
	 */
	private void ExecuteForBatch(ActionTypeEnum actionType, String moduleName, String operationName,
			DATA[] oldDataList, DATA[] newDataList, EventInfo eventInfo, INFO[] transitionInfoArray)
			throws NotFoundSignal, DuplicateNameSignal, InvalidStateTransitionSignal, FrameworkErrorSignal {
//		if(actionType == null)
//			actionType = actionType.action_default;

		ActionContainer container = actionContainers.get(actionType.toString());

		List<ActionFormat> actionList = container.getActionFormats(moduleName, operationName);

		HashMap<String, ValidationFormat> validationList = container.getValidationFormats(moduleName, operationName);

		// if null, get default container values.
		if (actionList == null || actionList.size() == 0) {
			logger.info(String.format(
					"Get DefaultActionContainer's Actions For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			actionList = defaultActionContainer.getActionFormats(moduleName, operationName);

			if (actionList == null || actionList.size() == 0)
				logger.warn(String.format("[Warn] Action is null ( ActionType[%s], moduleName[%s], operationName[%s]",
						actionType, moduleName, operationName));

		}
		if (validationList == null || validationList.size() == 0) {
			logger.info(String.format(
					"Get DefaultActionContainer's Validations For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			validationList = defaultActionContainer.getValidationFormats(moduleName, operationName);

			if (validationList == null || validationList.size() == 0)
				logger.warn(String.format(
						"[Warn] validationList is null ( ActionType[%s], moduleName[%s], operationName[%s]", actionType,
						moduleName, operationName));
		}

		for (ActionFormat action : actionList) {
			if (action.isDoAction()) {
				// get class
				Class actionClass = classFactory.get(action.getClassName());
				if (actionClass == null) {
					try {
						actionClass = Class.forName(action.getClassName());
						classFactory.put(action.getClassName(), actionClass);
					} catch (ClassNotFoundException ce) {
						logger.warn(String.format(
								"Fail to find Class for Action (moduleName[%s],ActionName[%s],ClassName[%s]",
								moduleName, action.getActionName(), action.getClassName()));
						continue;
					}
				}
				// get bundle
				Object beanInstance = ac.getBean(actionClass);
				try {
					Method method = actionClass.getMethod(action.getActionName(), oldDataList.getClass(), newDataList.getClass(),
							EventInfo.class, transitionInfoArray.getClass(), HashMap.class);
					method.invoke(beanInstance, oldDataList, newDataList, eventInfo, transitionInfoArray,
							validationList);
				} catch (IllegalArgumentException e) {
					throw new FrameworkErrorSignal("IllegalArgumentException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
									+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
				} catch (IllegalAccessException e) {
					throw new FrameworkErrorSignal("IllegalAccessException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
							+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
				} catch (InvocationTargetException e) {
					throw new FrameworkErrorSignal("InvocationTargetException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
							+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
				} catch (NoSuchMethodException e) {
					
					try {
						Method method = actionClass.getMethod(action.getActionName(), DataInfo[].class, DataInfo[].class,
								EventInfo.class, TransitionInfo[].class, HashMap.class);
						method.invoke(beanInstance, oldDataList, newDataList, eventInfo, transitionInfoArray,
								validationList);
					} catch (IllegalArgumentException e1) {
						throw new FrameworkErrorSignal("IllegalArgumentException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
										+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
					} catch (IllegalAccessException e1) {
						throw new FrameworkErrorSignal("IllegalAccessException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
					} catch (InvocationTargetException e1) {
						throw new FrameworkErrorSignal("InvocationTargetException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
					} catch (NoSuchMethodException e1) {
						
						throw new FrameworkErrorSignal(ExceptionKey.UnExpected_Exception, e1,
								"Can not found method '" + actionClass.getName() + "." + action.getActionName()
								+ "("+oldDataList.getClass()+","+ newDataList.getClass()+","+ EventInfo.class+","+ transitionInfoArray.getClass()+", HashMap)'.");
					}
				}
			}
		}

	}

	/**
	 * 
	 * @param actionType    in case of null, action_default. ex)action_default,
	 *                      action_NonHistory, action_ExceptAction, action_Glass,
	 *                      action_forBatch
	 * @param moduleName
	 * @param operationName
	 */
	private void ExecuteUpdateAtOnce(ActionTypeEnum actionType, String moduleName, String operationName,
			DATA[] oldDataList, EventInfo eventInfo, INFO transitionInfo, PersistentClass persistentClass)
			throws NotFoundSignal, DuplicateNameSignal, InvalidStateTransitionSignal, FrameworkErrorSignal {
//		if(actionType == null)
//			actionType = actionType.action_default;

		ActionContainer container = actionContainers.get(actionType.toString());

		List<ActionFormat> actionList = container.getActionFormats(moduleName, operationName);

		HashMap<String, ValidationFormat> validationList = container.getValidationFormats(moduleName, operationName);

		// if null, get default container values.
		if (actionList == null || actionList.size() == 0) {
			logger.info(String.format(
					"Get DefaultActionContainer's Actions For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			actionList = defaultActionContainer.getActionFormats(moduleName, operationName);

			if (actionList == null || actionList.size() == 0)
				logger.warn(String.format("[Warn] Action is null ( ActionType[%s], moduleName[%s], operationName[%s]",
						actionType, moduleName, operationName));

		}
		if (validationList == null || validationList.size() == 0) {
			logger.info(String.format(
					"Get DefaultActionContainer's Validations For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			validationList = defaultActionContainer.getValidationFormats(moduleName, operationName);

			if (validationList == null || validationList.size() == 0)
				logger.warn(String.format(
						"[Warn] validationList is null ( ActionType[%s], moduleName[%s], operationName[%s]", actionType,
						moduleName, operationName));
		}

		for (ActionFormat action : actionList) {
			if (action.isDoAction()) {
				// get class
				Class actionClass = classFactory.get(action.getClassName());
				if (actionClass == null) {
					try {
						actionClass = Class.forName(action.getClassName());
						classFactory.put(action.getClassName(), actionClass);
					} catch (ClassNotFoundException ce) {
						logger.warn(String.format(
								"Fail to find Class for Action (moduleName[%s],ActionName[%s],ClassName[%s]",
								moduleName, action.getActionName(), action.getClassName()));
						continue;
					}
				}
				// get bundle
				Object beanInstance = ac.getBean(actionClass);
				try {
					Method method = actionClass.getMethod(action.getActionName(), oldDataList.getClass(), EventInfo.class,
							transitionInfo.getClass(), PersistentClass.class, HashMap.class);
					method.invoke(beanInstance, oldDataList, eventInfo, transitionInfo, persistentClass,
							validationList);
				} catch (IllegalArgumentException e) {
					throw new FrameworkErrorSignal("IllegalArgumentException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
									+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
				} catch (IllegalAccessException e) {
					throw new FrameworkErrorSignal("IllegalAccessException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
									+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
				} catch (InvocationTargetException e) {
					throw new FrameworkErrorSignal("InvocationTargetException", e,
							"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
									+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
				} catch (NoSuchMethodException e) {
					try {
						Method method = actionClass.getMethod(action.getActionName(), DataInfo[].class, EventInfo.class,
								TransitionInfo.class, PersistentClass.class, HashMap.class);
						method.invoke(beanInstance, oldDataList, eventInfo, transitionInfo, persistentClass,
								validationList);
					} catch (IllegalArgumentException e1) {
						throw new FrameworkErrorSignal("IllegalArgumentException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
										+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
					} catch (IllegalAccessException e1) {
						throw new FrameworkErrorSignal("IllegalAccessException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
										+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
					} catch (InvocationTargetException e1) {
						throw new FrameworkErrorSignal("InvocationTargetException", e1,
								"Can not execute method '" + actionClass.getName() + "." + action.getActionName()
										+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
					} catch (NoSuchMethodException e1) {
						throw new FrameworkErrorSignal(ExceptionKey.UnExpected_Exception, e1,
								"Can not found method '" + actionClass.getName() + "." + action.getActionName()
										+ "("+oldDataList.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+","+ PersistentClass.class+", HashMap)'.");
					}
				}
			}
		}

	}

	public void ExecuteOneAction(String moduleName, String operationName, ActionFormat actionFormat,
			HashMap<String, ValidationFormat> validationList, DATA oldData, DATA newData, EventInfo eventInfo,
			INFO transitionInfo) throws NotFoundSignal, FrameworkErrorSignal {

		if (actionFormat.isDoAction()) {
			// get class
			Class actionClass = classFactory.get(actionFormat.getClassName());
			if (actionClass == null) {
				try {
					actionClass = Class.forName(actionFormat.getClassName());
					classFactory.put(actionFormat.getClassName(), actionClass);
				} catch (ClassNotFoundException ce) {
					logger.warn(String.format(
							"Fail to find Class for Action (moduleName[%s],operationName[%s], ActionName[%s]",
							moduleName, operationName, actionFormat.getActionName()));
					return;
				}
			}
			// get bundle
			Object beanInstance = ac.getBean(actionClass);
			try {
				Method method = actionClass.getMethod(actionFormat.getActionName(), oldData.getClass(), newData.getClass(),
						EventInfo.class, transitionInfo.getClass(), HashMap.class);
				method.invoke(beanInstance, oldData, newData, eventInfo, transitionInfo, validationList);
			} catch (IllegalArgumentException e) {
				throw new FrameworkErrorSignal("IllegalArgumentException", e,
						"Can not execute method '" + actionClass.getName() + "." + actionFormat.getActionName()
								+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
			} catch (IllegalAccessException e) {
				throw new FrameworkErrorSignal("IllegalAccessException", e,
						"Can not execute method '" + actionClass.getName() + "." + actionFormat.getActionName()
						+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
			} catch (InvocationTargetException e) {
				throw new FrameworkErrorSignal("InvocationTargetException", e,
						"Can not execute method '" + actionClass.getName() + "." + actionFormat.getActionName()
						+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
			} catch (NoSuchMethodException e) {
				try {
					Method method = actionClass.getMethod(actionFormat.getActionName(), DataInfo.class, DataInfo.class,
							EventInfo.class, TransitionInfo.class, HashMap.class);
					method.invoke(beanInstance, oldData, newData, eventInfo, transitionInfo, validationList);
				} catch (IllegalArgumentException e1) {
					throw new FrameworkErrorSignal("IllegalArgumentException", e1,
							"Can not execute method '" + actionClass.getName() + "." + actionFormat.getActionName()
									+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
				} catch (IllegalAccessException e1) {
					throw new FrameworkErrorSignal("IllegalAccessException", e1,
							"Can not execute method '" + actionClass.getName() + "." + actionFormat.getActionName()
							+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
				} catch (InvocationTargetException e1) {
					throw new FrameworkErrorSignal("InvocationTargetException", e1,
							"Can not execute method '" + actionClass.getName() + "." + actionFormat.getActionName()
							+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
				} catch (NoSuchMethodException e1) {
					throw new FrameworkErrorSignal(ExceptionKey.UnExpected_Exception, e1,
							"Can not found method '" + actionClass.getName() + "." + actionFormat.getActionName()
							+ "("+oldData.getClass()+","+ newData.getClass()+","+ EventInfo.class+","+ transitionInfo.getClass()+")'.");
				}
			}
		}

	}

	public ActionFormatAdditionalInfo getActionLists(String actionTypeName, String moduleName, String operationName) {
		return getActionLists(getActionType(actionTypeName), moduleName, operationName);
	}

	private ActionFormatAdditionalInfo getActionLists(ActionTypeEnum actionType, String moduleName,
			String operationName) {
//		if(actionType == null)
//			actionType = actionType.action_default;

		ActionContainer container = actionContainers.get(actionType.toString());

		List<ActionFormat> actionList = container.getActionFormats(moduleName, operationName);

		HashMap<String, ValidationFormat> validationList = container.getValidationFormats(moduleName, operationName);

		// if null, get default container values.
		if (actionList == null) {
			logger.info(String.format(
					"Get DefaultActionContainer's Actions For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			actionList = defaultActionContainer.getActionFormats(moduleName, operationName);
		}
		if (validationList == null) {
			logger.info(String.format(
					"Get DefaultActionContainer's Validations For ActionType[%s], moduleName[%s], operationName[%s]",
					actionType, moduleName, operationName));
			validationList = defaultActionContainer.getValidationFormats(moduleName, operationName);
		}

		return new ActionFormatAdditionalInfo(actionList, validationList);
	}

	public HashMap<String, String> getBehaviorFileList() {
		return getActionFileList();
	}

	public void setBehaviorFileList(HashMap<String, String> behaviorFileList) {
		this.setActionFileList(behaviorFileList);
	}

	public HashMap<String, String> getActionFileList() {
		return actionFileList;
	}

	public void setActionFileList(HashMap<String, String> actionFileList) {
		this.actionFileList = actionFileList;
	}

	public enum ActionTypeEnum {
		action_default, action_exceptAction, action_glass, action_noneHistory, action_forBatch
	}

	public static ActionTypeEnum getActionType(String actionType) {
		if (StringUtils.isEmpty(actionType))
			return ActionTypeEnum.action_default;
		else if (actionType.equalsIgnoreCase(ActionTypeEnum.action_default.toString())) {
			return ActionTypeEnum.action_default;
		} else if (actionType.equalsIgnoreCase(ActionTypeEnum.action_exceptAction.toString())) {
			return ActionTypeEnum.action_exceptAction;
		} else if (actionType.equalsIgnoreCase(ActionTypeEnum.action_glass.toString())) {
			return ActionTypeEnum.action_glass;
		} else if (actionType.equalsIgnoreCase(ActionTypeEnum.action_noneHistory.toString())) {
			return ActionTypeEnum.action_noneHistory;
		} else if (actionType.equalsIgnoreCase(ActionTypeEnum.action_forBatch.toString())) {
			return ActionTypeEnum.action_forBatch;
		} else
			return ActionTypeEnum.action_default;
	}

//	public List<String> getBundleSymbolicNameList() {
//		return bundleSymbolicNameList;
//	}
//
//	public void setBundleSymbolicNameList(List<String> bundleSymbolicNameList) {
//		this.bundleSymbolicNameList = bundleSymbolicNameList;
//	}

	public HashMap<String, ActionContainer> getActionContainers() {
		return actionContainers;
	}

}