package com.cim.idm.framework.event;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.cim.idm.framework.IDMFrameServiceProxy;
import com.cim.idm.framework.esb.IListener;
import com.cim.idm.framework.fos.idmflow.BpelExecutorService;
import com.cim.idm.framework.util.service.ServiceUtil;



public abstract class AbstractMessageEventTemplate implements ApplicationContextAware, IMessageEvent
{
	public ApplicationContext	applicationContext;
	private static Log			log	= LogFactory.getLog(AbstractMessageEventTemplate.class);
	private List<String>		listenBeanNames;
	private MessageEventAdaptor messageEventAdaptor = null;
	
	public AbstractMessageEventTemplate()
	{
	}

	public List<String> getListenBeanNameList()
	{
		return listenBeanNames;
	}

	public void setListenBeanNameList(List<String> beanNameList)
	{
		this.listenBeanNames = beanNameList;
	}

	public void setApplicationContext(ApplicationContext arg0) throws BeansException
	{
		applicationContext = arg0;
		init();
	}

	private void init()
	{
		if (this.listenBeanNames == null || this.listenBeanNames.size() == 0)
		{
			log.warn("No registered targetServiceBeans for MessageEvent");
			return;
		}
		
		try
		{
			Map map = this.applicationContext.getBeansOfType(this.getClass());
			Set set = map.keySet();
			Object[] hmKeys = set.toArray();

			try {
				messageEventAdaptor = (MessageEventAdaptor) this.applicationContext.getBean(ServiceUtil.formatSimpleClassName(MessageEventAdaptor.class));
			} catch (Exception e) {}

			if (messageEventAdaptor == null)			
				messageEventAdaptor = (MessageEventAdaptor) applicationContext
					.getBean(MessageEventAdaptor.class);

			for (Object obj : hmKeys)
			{
				for (String name : this.listenBeanNames)
				{
					messageEventAdaptor.registerEvent(obj.toString(), name);
					try {
						IListener listener = (IListener)this.applicationContext.getBean(name);
						messageEventAdaptor.registerListener(name,listener);
					}catch(Exception e) {
						log.error(e);
					}
				}
			}
		} 
		catch (Exception e)
		{
			log.error(e, e);
		}
	}

	public void execute(Object[] data)
	{
		try
		{
//			if(IDMFrameServiceProxy.isPossibleIDMFlow())
//				((com.cim.idm.framework.fos.idmflow.BpelExecutorService) BundleUtil.waitForService(com.cim.idm.framework.fos.idmflow.BpelExecutorService.class.getName())).executeProcess(data, null);
//			else
			Map<String, BpelExecutorService> bpelExecutorMap ;
			try {
				bpelExecutorMap = applicationContext.getBeansOfType(BpelExecutorService.class);;
			}catch(Exception e) {
				bpelExecutorMap = applicationContext
						.getBeansOfType(BpelExecutorService.class);
			}
			for(Entry<String, BpelExecutorService> entry :bpelExecutorMap.entrySet()) {
				String bpelExecutorServiceBean = entry.getKey();
				BpelExecutorService bpelExecutorService = entry.getValue();
				log.debug("Find BpelExecutorService 's subclass Bean :" + bpelExecutorServiceBean);
				bpelExecutorService.executeProcess(data, null);
				break;
			}
			
		} 
		catch (Exception e)
		{
			log.error(e, e);
		}		
	}
	
	public void execute(Object data) {
		Map<String, BpelExecutorService> bpelExecutorMap ;
		try {
			bpelExecutorMap = applicationContext.getBeansOfType(BpelExecutorService.class);;
		}catch(Exception e) {
			bpelExecutorMap = applicationContext
					.getBeansOfType(BpelExecutorService.class);
		}
		for(Entry<String, BpelExecutorService> entry :bpelExecutorMap.entrySet()) {
			String bpelExecutorServiceBean = entry.getKey();
			BpelExecutorService bpelExecutorService = entry.getValue();
			log.debug("Find BpelExecutorService 's subclass Bean :" + bpelExecutorServiceBean);
			bpelExecutorService.executeProcess(data);
			break;
		}
		
	}
}
