package com.cim.idm.framework.fos.idmflow;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.jar.JarFile;

import javax.xml.namespace.QName;

import org.apache.commons.collections4.ListUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.logging.log4j.core.util.OptionConverter;

import com.cim.idm.flowpackage.core.activity.BpelProcess;
import com.cim.idm.flowpackage.repository.AbstractBpelRepository;
import com.cim.idm.framework.util.file.FileListener;
import com.cim.idm.framework.util.file.FileListenerEvent;
import com.cim.idm.framework.util.file.FileUtil;
import com.cim.idm.framework.util.file.FileWatcher;
import com.cim.idm.framework.util.resource.ResourceUtil;
import com.cim.idm.framework.util.service.IDMFrameProperties;

public class BpelExRepository extends AbstractBpelRepository implements ExRepository, FileListener
{

	private static Log					log					= LogFactory.getLog(BpelExRepository.class);
	private boolean						useBpelsInInnerDirs	= false;
	private Map<String, Long>			bpelMapList			= new HashMap<String, Long>();
	
//	private String						bpelRootPath		= "/META-INF/bpels";
	private String						bpelRootPath		= "";
	private List<String>				bpelRootPathList	= null;
	private String						bpelJarEntryPath	= "";
	private String						rootBpelName		= "dispatching.bpel";
//	private List<String>				bundleSymbolicNameList;
	private Map<String, FileWatcher>	FileWatcherList		= new HashMap<String, FileWatcher>();
	
	private static String				DEFAULT_TARGETNAMESPACE	= "http://bpel.boe.com/bpelj/"; 						
	
	private boolean bpelWatch 								= true;
	private int watchInterval 								= 10;

	public BpelExRepository()
	{
	}

	public String getBpelRootPath()
	{
		return bpelRootPath;
	}

	public void setBpelRootPath(String bpelRootPath)
	{
		this.bpelRootPath = OptionConverter.substVars(bpelRootPath, null);
	}

	public void setBpelJarEntryPath(String bpelJarEntryPath) {
		this.bpelJarEntryPath = bpelJarEntryPath;
	}

	public List<String> getBpelRootPathList()
	{
		return bpelRootPathList;
	}

	public void setBpelRootPathList(List<String> bpelRootPathList)
	{
		String[] bpelDir = new String[bpelRootPathList.size()];
		for (int i = 0; i < bpelRootPathList.size(); i++)
		{
			bpelDir[i] = OptionConverter.substVars(bpelRootPathList.get(i).toString(), null);
		}
		bpelRootPathList.clear();

		for (int i = 0; i < bpelDir.length; i++)
		{
			bpelRootPathList.add(bpelDir[i]);
		}
		this.bpelRootPathList = bpelRootPathList;
	}

	public void setRepositoryName(String name)
	{
		super.setRepositoryName(System.getProperty("Process"));
	}

	public String getRootBpelName()
	{
		return rootBpelName;
	}

	public void setRootBpelName(String rootBpelName)
	{
		this.rootBpelName = rootBpelName;
	}

	public int getWFThreadSize()
	{
		return this.getBpelProcessManager().getWFThreadSize();
	}

	public boolean isUseBpelsInInnerDirs()
	{
		return useBpelsInInnerDirs;
	}

	public void setUseBpelsInInnerDirs(boolean useBpelsInInnerDirs)
	{
		this.useBpelsInInnerDirs = useBpelsInInnerDirs;
	}
	
	public void setBpelWatch( boolean bpelWatch )
	{
		this.bpelWatch = bpelWatch;
	}
	
	public void setWatchInterval( int watchInterval )
	{
		this.watchInterval = watchInterval;
	}
	
	public void init() throws Exception
	{
		try
		{
			read();
		} 
		catch (Throwable e)
		{
			log.error(e, e);
		}
	}

	public void shutdown()
	{
		Iterator<BpelProcess> iter = getRegisteredBpelProcesses().values().iterator();

		while (iter.hasNext())
		{
			this.unregisterStaticMBean(iter.next());
		}

		getBpelProcessManager().unregisterMBeanBpelRepository(this);
	}

	@SuppressWarnings("unchecked")
	private void readBpels(String aBpelRootPath, List<File> bpelAllList, List<String> failedList )
	{
		List<File> bundleBpelList = new ArrayList<File>();

		try
		{
			List<File> bpelList = FileUtil.getFileList(new File(aBpelRootPath), ".BPEL", this.useBpelsInInnerDirs);
			log.info("Reading Bpel Files(" + bpelList.size() + ") from " + aBpelRootPath );
			bundleBpelList = (List<File>) ListUtils.sum(bpelList, bundleBpelList);
		} 
		catch (Exception e)
		{}

		if ( bundleBpelList == null || bundleBpelList.size() == 0 )
			log.warn("Could not find application service files [" + aBpelRootPath + "]");

		for (int i = 0; i < bundleBpelList.size(); i++)
		{
			String path = ((File) bundleBpelList.get(i)).getAbsolutePath();
			
			try
			{
				this.addBpelProcess(path);
			}
			catch( Exception ex )
			{
				failedList.add( path );
				log.error( "Failed reading bpel file. [" + path + "]", ex );
			}
		}
		
		for ( File file : bundleBpelList )
		{
			bpelAllList.add( file );
		}
	}
	
	private void readJarBpels(JarFile jar, String aBpelJarEntryPath, List<File> bpelAllList, List<String> failedList )
	{
		List<File> bundleBpelList = new ArrayList<File>();

		try
		{
			List<org.w3c.dom.Document> docList = ResourceUtil.getBundleBpels(jar, aBpelJarEntryPath, bundleBpelList, failedList );
				
			if (docList.size() > 0)
			{
				for (int j = 0; j < docList.size(); j++)
				{
					this.addBpelProcess(docList.get(j));
				}
				return;
			}
			else
				log.warn("Could not find application bpel files ["+jar.getName()+":" + aBpelJarEntryPath + "]");
		} 
		catch (Exception e)
		{
//			if (bundle == null)
//					bundle = bundleContext.getBundle();
				
			throw e;
				
		}

		if ( bundleBpelList == null || bundleBpelList.size() == 0 )
			log.warn("Could not find application service files [" +jar.getName()+":" + aBpelJarEntryPath + "]");

		for (int i = 0; i < bundleBpelList.size(); i++)
		{
			String path = ((File) bundleBpelList.get(i)).getAbsolutePath();
			
			try
			{
				this.addBpelProcess(path);
			}
			catch( Exception ex )
			{
				failedList.add( path );
				log.error( "Failed reading bpel file. [" + path + "]", ex );
			}
		}
		
		for ( File file : bundleBpelList )
		{
			bpelAllList.add( file );
		}
	}

	public void read() throws RuntimeException
	{
		super.read();

		List<File> bpelAllList = new ArrayList<File>();
		List<String> failedList = new ArrayList<String>();

		if (this.bpelRootPathList == null)
		{
			JarFile jar = null;
			if(bpelRootPath.startsWith(".jar")) {
				try {
					jar = new JarFile(bpelRootPath);
					readJarBpels(jar, bpelJarEntryPath, bpelAllList, failedList);
				} catch (IOException e) {
					log.error("The JAR File:" + bpelRootPath + " Not found, when detect BPEL Files");
					log.error(e);
				}
				
			}else {
				readBpels( this.bpelRootPath, bpelAllList, failedList );
			}
			if ( this.bpelWatch )
				startWatcher( this.bpelRootPath );
		}
		else
		{
			for (int i = 0; i < bpelRootPathList.size(); i++)
			{
				JarFile jar = null;
				if(this.bpelRootPathList.get(i).startsWith(".jar")) {
					try {
						jar = new JarFile(this.bpelRootPathList.get(i));
						readJarBpels(jar, bpelJarEntryPath, bpelAllList, failedList);
					} catch (IOException e) {
						log.error("The JAR File:" + this.bpelRootPathList.get(i) + " Not found, when detect BPEL Files");
						log.error(e);
					}
					
				}else {
					readBpels( this.bpelRootPathList.get(i), bpelAllList, failedList );
				}
				if ( this.bpelWatch )
					startWatcher( this.bpelRootPathList.get(i));
			}
		}
		
		logBpelLoadResult( bpelAllList, failedList );
	}
	
	private void logBpelLoadResult( List<File> bpelAllList, List<String> failedList )
	{
		log.info( "Completed reading " + ( bpelAllList.size() - failedList.size()) + " Bpel files. " + "Total=" + bpelAllList.size() + ", Fail=" + failedList.size());
		
		StringBuilder builder = new StringBuilder();
		
		if ( failedList.size() > 0 )
		{
			for ( String name : failedList )
				builder.append( name ).append( ", " );
			
			log.warn( "Failed Bpel files : " + builder.substring( 0, builder.length() - 2 ));
		}
	}

	private synchronized void removeBpelProcess(String bpelfile) throws Exception
	{
		File file = new File(bpelfile);
		String path = file.getAbsolutePath();
		bpelMapList.remove(path);
		QName qname = new QName(DEFAULT_TARGETNAMESPACE, file.getName());
		this.removeBpelProcess(qname);
	}

	private synchronized void addBpelProcess(String bpelfile) throws Exception
	{
		File file = new File(bpelfile);
		String path = file.getAbsolutePath();

		if (bpelMapList.containsKey(path))
		{
			if (file.lastModified() > bpelMapList.get(path).longValue())
				bpelMapList.remove(path);
			else
				return;
		}

		org.w3c.dom.Document doc = xutil4j.xml.DomUtils.load(path, true);
		//		try {
		//			this.removeBpelProcess(getQName(doc));		
		//		} catch (Exception e) {}

		this.addBpelProcess(doc);
		bpelMapList.put(path, file.lastModified());
	}

	public void reload() throws RuntimeException
	{
		removeAll();
		read();
	}

	public void removeAll()
	{
		this.getRegisteredBpelProcesses().clear();
		super.removeAll();
	}

	private void startWatcher(String watchDir)
	{
		try
		{
			FileWatcher fileWatcher = FileWatcherList.get(watchDir);
			if (fileWatcher != null)
			{
				fileWatcher.stopWatching();
				fileWatcher.removeFileListener(this);
				FileWatcherList.remove(watchDir);
			}

			fileWatcher = new FileWatcher(watchDir, ".BPEL", this.useBpelsInInnerDirs);
			fileWatcher.setInterval(this.watchInterval); // 10 seconds
			fileWatcher.addFileListener(this);
			FileWatcherList.put(watchDir, fileWatcher);
		} catch (Exception e)
		{
			log.warn("Could not create BpelWatcher");
		}
	}

	public void onFileChanged(FileListenerEvent e)
	{
		try
		{
			if (e.getEventType().equalsIgnoreCase(FileListenerEvent.FILE_EVENT_TYPE_ADD)
				|| e.getEventType().equalsIgnoreCase(FileListenerEvent.FILE_EVENT_TYPE_MODIFY))
				addBpelProcess(e.getFile());
			else if (e.getEventType().equalsIgnoreCase(FileListenerEvent.FILE_EVENT_TYPE_DELETE))
				removeBpelProcess(e.getFile());
		} catch (Exception ex)
		{
			log.error(ex);
		}
	}
}
