package com.cim.idm.framework.fos.idmflow;

import javax.management.MBeanServer;
import javax.xml.namespace.QName;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Document;
import org.json.JSONObject;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.jmx.support.MBeanServerFactoryBean;

import com.cim.idm.flowpackage.core.BpelExecutionContext;
import com.cim.idm.flowpackage.core.BpelProcessContext;
import com.cim.idm.flowpackage.core.BpelProcessManager;
import com.cim.idm.flowpackage.core.activity.BpelProcess;
import com.cim.idm.flowpackage.expression.BpelExpression;
import com.cim.idm.flowpackage.expression.IdmFlowFunction;
import com.cim.idm.framework.IDMFrameServiceProxy;
import com.cim.idm.framework.util.msg.DocumentMessageUtil;
import com.cim.idm.framework.util.msg.JsonMessageUtil;
import com.cim.idm.framework.util.service.IDMFrameProperties;
import com.cim.idm.framework.util.service.ServiceUtil;


 public abstract class BpelExecutorService implements BpelProcessContext
 {

	public static final String			DEFAULT_TARGETNAMESPACE	= "http://bpel.boe.com/bpelj/";
	protected static Log				log						= LogFactory.getLog(BpelExecutorService.class);

	protected BpelExRepository			bpelRepository;
	protected BpelExecutionEventAdaptor	bpelExecutionEventAdaptor;
	protected String					rootBpelName;
	protected boolean					parallelMode			= true;
	protected String					bpelNameSpaceURI;
	
	BpelProcessManager bpelProcessManager = null;
	BpelExpression bpelExpression = null;
	
	public void setParallelMode(boolean parallelMode)
	{
		this.parallelMode = parallelMode;
	}

	public String getRootBpelName()
	{
		return rootBpelName;
	}

	public void setRootBpelName(String rootBpelName)
	{
		this.rootBpelName = rootBpelName;
	}

	public BpelExRepository getBpelRepository()
	{
		return bpelRepository;
	}

	public void setBpelRepository(BpelExRepository bpelRepository)
	{
		this.bpelRepository = bpelRepository;
	}

	public BpelExecutionEventAdaptor getBpelExecutionEventAdaptor()
	{
		return bpelExecutionEventAdaptor;
	}

	public void setBpelExecutionEventAdaptor(BpelExecutionEventAdaptor bpelExecutionEventAdaptor)
	{
		this.bpelExecutionEventAdaptor = bpelExecutionEventAdaptor;
	}

	public synchronized Object getBean(String beanId)
	{
//		Object obj = this.applicationContext.getBean(beanId);
//		if (obj == null)
		return IDMFrameServiceProxy.getApplicationContext().getBean(beanId);
//		else
//			return obj;
	}

	public BpelExpression getBpelExpression()
	{
		if(bpelExpression == null)
		{
			try {
				bpelExpression = (BpelExpression)IDMFrameServiceProxy.getApplicationContext().getBean(IdmFlowFunction.class.getSimpleName());
			}catch(Exception e)
			{
				bpelExpression = (BpelExpression) IDMFrameServiceProxy.getApplicationContext().getBean(IdmFlowFunction.class);
			}
		}
		return bpelExpression;
	}
	
	public BpelProcessManager getBpelProcessManager()
	{
		if(bpelProcessManager == null)
		{	try
			{
				bpelProcessManager = (BpelProcessManager)IDMFrameServiceProxy.getApplicationContext().getBean(BpelProcessManager.class.getSimpleName());
			}catch(Exception e)
			{
				bpelProcessManager = (BpelProcessManager) IDMFrameServiceProxy.getApplicationContext().getBean(BpelProcessManager.class);
			}
		}
		return bpelProcessManager;
	}

	public MBeanServer getMBeanServer()
	{
		try
		{
			MBeanServer mBeanServer = (MBeanServer)IDMFrameServiceProxy.getApplicationContext().getBean(MBeanServerFactoryBean.class.getSimpleName());
			if (mBeanServer == null)
				return (MBeanServer) IDMFrameServiceProxy.getApplicationContext().getBean(MBeanServerFactoryBean.class);
			else 
				return mBeanServer;
		} catch (Exception ex)
		{
			return null;
		}
	}

	public BpelProcess newBpelProcess(QName qname)
	{
		BpelProcess bpelProcess = this.bpelRepository.newBpelProcess(qname);
		bpelProcess.getBpelExecutionContext().setBpelRepositoryName(this.bpelRepository.getRepositoryName());
		bpelProcess.getBpelExecutionContext().setBpelExpression(this.getBpelExpression());
		bpelProcess.setBpelProcessContext(this);
		return bpelProcess;
	}

	public void executeProcess(Object[] arguments)
	{
		executeProcess(arguments, this.getBpelRepository().getRootBpelName());
	}
	
	public void executeProcess(Object argument)
	{
		executeProcess(argument, this.getBpelRepository().getRootBpelName());
	}
	
	public void executeProcess(Object argument, String bpelName)
	{
		BpelProcess process = null;
		try
		{
			if (bpelName == null)
			{

				if (argument instanceof Document)
				{
					String bpel = DocumentMessageUtil.getBpelName((Document) argument);
					if (bpel != null && bpel.length() > 0)
					{
						if (!bpel.toLowerCase().endsWith(".bpel"))
							bpelName = bpel + ".bpel";
						else
							bpelName = bpel;
					}
				}else if(argument instanceof JSONObject) {
					String bpel = JsonMessageUtil.getBpelName((JSONObject) argument);
					if (bpel != null && bpel.length() > 0)
					{
						if (!bpel.toLowerCase().endsWith(".bpel"))
							bpelName = bpel + ".bpel";
						else
							bpelName = bpel;
					}
				}

				if (bpelName == null)
					bpelName = this.getBpelRepository().getRootBpelName();
			}
			QName name ;
			if(StringUtils.isEmpty(bpelNameSpaceURI))
				name = new QName(DEFAULT_TARGETNAMESPACE, bpelName);
			else
				name = new QName(bpelNameSpaceURI,bpelName);
			process = newBpelProcess(name);

/*			Object obj = BundleUtil.getServiceByBeanName("Modeler");
			if (obj != null)
				process.addBpelActivityListener((BpelActivityListener) obj);
			else
				process.addBpelActivityListener(getBpelExecutionEventAdaptor());
*/
			process.addBpelActivityListener(getBpelExecutionEventAdaptor());
//			Object[] arguments = new Object[1];
//			arguments[0] = argument;
			process.execute(new Object[] {argument}, parallelMode);
//			process.execute(arguments);

		} catch (Exception ex)
		{
			log.error("Could not execute " + bpelName, ex);
			getBpelExecutionEventAdaptor().onException(	process, ex);
		}
	}

	public void executeProcess(Object[] arguments, String bpelName)
	{
		BpelProcess process = null;
		try
		{
			if (bpelName == null)
			{
				for (int i = 0; i < arguments.length; i++)
				{
					if (arguments[i] instanceof Document)
					{
						String bpel = DocumentMessageUtil.getBpelName((Document) arguments[i]);
						if (bpel != null && bpel.length() > 0)
						{
							if (!bpel.toLowerCase().endsWith(".bpel"))
								bpelName = bpel + ".bpel";
							else
								bpelName = bpel;
							break;
						}
					}else if(arguments[i] instanceof JSONObject) {
						
					}
				}
				if (bpelName == null)
					bpelName = this.getBpelRepository().getRootBpelName();
			}
			QName name ;
			if(StringUtils.isEmpty(bpelNameSpaceURI))
				name = new QName(DEFAULT_TARGETNAMESPACE, bpelName);
			else
				name = new QName(bpelNameSpaceURI,bpelName);
			process = newBpelProcess(name);

/*			Object obj = BundleUtil.getServiceByBeanName("Modeler");
			if (obj != null)
				process.addBpelActivityListener((BpelActivityListener) obj);
			else
				process.addBpelActivityListener(getBpelExecutionEventAdaptor());
*/
			process.addBpelActivityListener(getBpelExecutionEventAdaptor());
			
			process.execute(arguments, parallelMode);

		} catch (Exception ex)
		{
			log.error("Could not execute " + bpelName, ex);
			getBpelExecutionEventAdaptor().onException(	process, ex);
		}
	}

	public void executeWF(Document document, String bpelName)
	{
		Object[] arguments = new Object[] { document };
		this.executeWF(arguments, bpelName, false);
	}

	public void executeWF(Object[] arguments, String bpelName, boolean parallelMode)
	{

		BpelProcess process = null;
		try
		{
			if (bpelName == null)
			{
				for (int i = 0; i < arguments.length; i++)
				{
					if (arguments[i] instanceof Document)
					{
						String bpel = DocumentMessageUtil.getBpelName((Document) arguments[i]);
						if (bpel != null && bpel.length() > 0)
						{
							if (!bpel.toLowerCase().endsWith(".bpel"))
								bpelName = bpel + ".bpel";
							else
								bpelName = bpel;
							break;
						}
					}
				}

				if (bpelName == null)
					bpelName = this.getBpelRepository().getRootBpelName();
			}

			QName name = new QName(DEFAULT_TARGETNAMESPACE, bpelName);
			process = newBpelProcess(name);

			process.setParent(BpelExecutionContext.currentBpelExecutionContext().getRunningActivity());
			process.setSuperProcess(BpelExecutionContext.currentBpelExecutionContext().getBpelProcess());
			process.addBpelActivityListener(BpelExecutionContext.currentBpelExecutionContext().getRunningActivity());

			process.execute(arguments, parallelMode, false, BpelExecutionContext.currentBpelExecutionContext()
					.getBpelProcess()
					.getOriginatorBpelProcess());

		} catch (Exception ex)
		{
			log.error("Could not execute " + bpelName, ex);
			getBpelExecutionEventAdaptor().onException(process, ex);
		}
	}
}
