package com.cim.idm.framework.fos.idmflow;

import java.net.InetAddress;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Document;
import org.jdom.Element;

import com.cim.idm.flowpackage.core.activity.BpelProcess;
import com.cim.idm.framework.esb.IRequester;
import com.cim.idm.framework.util.msg.DocumentMessageUtil;
import com.cim.idm.framework.util.xml.JdomUtils;

public class BpelMonitor
{

	private static Log	log	= LogFactory.getLog(BpelMonitor.class);
	private String		IPAddress;
	private IRequester	bpelMonitorRequester;
	private String		bootCompleteTime;
	private boolean		autoManaged;

	public boolean isAutoManaged()
	{
		return autoManaged;
	}

	public void setAutoManaged(boolean autoManaged)
	{
		this.autoManaged = autoManaged;
	}

	public IRequester getBpelMonitorRequester()
	{
		return bpelMonitorRequester;
	}

	public void setBpelMonitorRequester(IRequester bpelMonitorRequester)
	{
		this.bpelMonitorRequester = bpelMonitorRequester;
	}

	public BpelMonitor()
	{
		try
		{
			IPAddress = InetAddress.getLocalHost().getHostAddress();
		} catch (Exception e)
		{}
	}

	public void sendMonitorData(BpelProcess process, Document doc, boolean isOK)
	{
		try
		{
			Document monitorDoc = createMonitorData(process, doc, isOK);
			bpelMonitorRequester.send(JdomUtils.toString(monitorDoc));
		} catch (Exception e)
		{
			log.info("Could not monitor data, but not critical error");
		}
	}

	public void sendMonitorData(BpelProcess process, List<Element> history, Document doc, boolean isOK)
	{
		try
		{
			Document monitorDoc = createMonitorData(process, history, doc, isOK);
			bpelMonitorRequester.send(JdomUtils.toString(monitorDoc));
		} catch (Exception e)
		{
			log.info("Could not monitor data, but not critical error");
		}
	}

	private Document createHeaderMessage(String command)
	{
		Element envelop = new Element("monitoring");
		Document document = new Document(envelop);

		Element header = new Element("header");
		envelop.addContent(header);

		Element element = new Element("command");
		element.setText(command);
		header.addContent(element);

		element = new Element("ip");
		element.setText(this.IPAddress);
		header.addContent(element);

		try
		{
			element = new Element("appgroup");
			element.setText(System.getProperty("ProcessType"));
			header.addContent(element);
		} catch (Exception e)
		{}

		try
		{
			element = new Element("appname");
			element.setText(System.getProperty("Process"));
			header.addContent(element);
		} catch (Exception e)
		{}

		return document;
	}

	public Document createReloadBpel()
	{
		return createHeaderMessage("reloadbpels");
	}

	public Document createReloadSql()
	{
		return createHeaderMessage("reloadsql");
	}

	public Document createReloadAll()
	{
		return createHeaderMessage("reloadall");
	}

	public Document createBootCompleted(String listenerType, boolean nt, boolean jdbc, String threadMode)
	{
		Document doc = createHeaderMessage("bootcompleted");

		Element body = new Element("body");
		doc.getRootElement().addContent(body);

		Element element = new Element("starttime");
		String currentTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date());
		element.setText(currentTime);
		body.addContent(element);

		element = new Element("listenmode");
		element.setText(listenerType);
		body.addContent(element);

		element = new Element("threadmode");
		element.setText(threadMode);
		body.addContent(element);

		element = new Element("nt.autocommit");
		element.setText(String.valueOf(nt));
		body.addContent(element);

		element = new Element("jdbc.autocommit");
		element.setText(String.valueOf(jdbc));
		body.addContent(element);

		bootCompleteTime = currentTime;
		return doc;
	}

	public Document createShutdown()
	{
		Document doc = createHeaderMessage("shutdown");

		Element body = new Element("body");
		doc.getRootElement().addContent(body);

		Element element = new Element("starttime");
		element.setText(bootCompleteTime);
		body.addContent(element);

		element = new Element("endtime");
		String currentTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date());
		element.setText(currentTime);
		body.addContent(element);

		return doc;
	}

	public Document createMonitorData(BpelProcess process, List<Element> history, Document doc, boolean isOK)
	{
		Document document = createMonitorData(process, doc, isOK);
		Element element = new Element("history");
		for (int i = 0; i < history.size(); i++)
		{
			element.addContent(history.get(i));
		}
		document.getRootElement().addContent(element);
		return document;
	}

	public Document createMonitorData(BpelProcess process, Document doc, boolean isOK)
	{
		Document document = createHeaderMessage("bpelexecuted");

		Element body = new Element("body");
		document.getRootElement().addContent(body);

		Element element = new Element("starttime");
		String startTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(process.getStartTime());
		element.setText(startTime);
		body.addContent(element);

		element = new Element("endtime");
		String endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(process.getEndTime());
		element.setText(endTime);
		body.addContent(element);

		element = new Element("bpelname");
		element.setAttribute("id", process.getBpelProcess().getId());
		element.setAttribute("procid", process.getBpelProcess().getProcId());
		element.setText(process.getBpelProcess().getName());
		body.addContent(element);

		element = new Element("success");
		element.setText(String.valueOf(isOK));
		body.addContent(element);

		element = new Element("elapsedtime");
		element.setText(String.valueOf(process.getEndTime().getTime() - process.getStartTime().getTime()));
		body.addContent(element);

		//		String equipmentName = "";
		//		String lotName = "";
		//		
		//		String listen_process = MessageUtil.getHeaderItemValue(doc, MessageUtil.Header_Listen_Process);
		//		if (listen_process.startsWith("EIS"))
		//		{
		//			try {
		//				Element element2 = JdomUtils.getNode(doc, "//Body/DATA");
		//				equipmentName = element2.getAttributeValue("EQUIPMENT_ID");
		//				if (equipmentName == null) equipmentName = "";
		//			} catch (Exception e) {}
		//			try {			
		//				Element element2 = JdomUtils.getNode(doc, "//LOT");
		//				lotName = element2.getAttributeValue("LOT_ID");				
		//			} catch (Exception e) {}
		//		}
		//		else {
		//			equipmentName = MessageUtil.getBodyItemValue(doc, "MachineId");
		//			lotName = MessageUtil.getBodyItemValue(doc, "LotId");
		//		}
		//				
		//		element = new Element("equipmentname");       		
		//		element.setText(equipmentName);	
		//		body.addContent(element);
		//		
		//		element = new Element("lotname");       		
		//		element.setText(lotName);	
		//		body.addContent(element);

		element = new Element("errorcode");
		element.setText(DocumentMessageUtil.getResultItemValue(doc, DocumentMessageUtil.Result_ReturnCode));
		body.addContent(element);

		element = new Element("errormessage");
		element.setText(DocumentMessageUtil.getResultItemValue(doc, DocumentMessageUtil.Result_ErrorMessage));
		body.addContent(element);

		/*
		 * try { element = new Element("xmldata");
		 * element.addContent(doc.getRootElement().cloneContent()); body.addContent(element);
		 * //element.setContent(new CDATA(JdomUtils.toString(doc))); //envelop.addContent(element);
		 * } catch (Exception e) { e.printStackTrace(); }
		 */
		return document;
	}

	public static String getStandardTime(Date time)
	{
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(time);
	}

}
