package com.cim.idm.framework.orm;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import org.springframework.dao.TypeMismatchDataAccessException;
import org.springframework.jdbc.IncorrectResultSetColumnCountException;
import org.springframework.jdbc.core.SingleColumnRowMapper;

public class NullReplaceableSingleRowMapper<T> extends SingleColumnRowMapper<T> 
{
	private Class<T> requiredType;
	
	public NullReplaceableSingleRowMapper()
	{}
	
	public NullReplaceableSingleRowMapper( Class<T> requiredType )
	{
		this.requiredType = requiredType;
	}
	
	public void setRequiredType( Class<T> requiredType )
	{
		this.requiredType = requiredType;
	}
	
	public T mapRow( ResultSet rs, int rowNum ) throws SQLException
	{
		// Validate column count.
		ResultSetMetaData rsmd = rs.getMetaData();
		int nrOfColumns = rsmd.getColumnCount();
		if ( nrOfColumns != 1 ) { throw new IncorrectResultSetColumnCountException( 1, nrOfColumns ); }

		// Extract column value from JDBC ResultSet.
		Object result = getColumnValue( rs, 1, this.requiredType );
		if ( result != null && this.requiredType != null && !this.requiredType.isInstance( result ) )
		{
			// Extracted value does not match already: try to convert it.
			try
			{
				return (T)convertValueToRequiredType( result, this.requiredType );
			}
			catch ( IllegalArgumentException ex )
			{
				throw new TypeMismatchDataAccessException( "Type mismatch affecting row number " + rowNum + " and column type '"
						+ rsmd.getColumnTypeName( 1 ) + "': " + ex.getMessage() );
			}
		}
		else if ( result == null && String.class.equals( this.requiredType ))
			result = "";
		
		return (T)result;
	}
	
}
