package com.cim.idm.framework.orm;

import java.sql.Date;
import java.sql.Timestamp;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;


import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;

import com.cim.idm.framework.orm.info.DataInfo;

/*
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.orm
 *  NAME    : SQLLogUtil.java
 *  TYPE    : JAVA
 *  DESCRIPTION :
 *
 ****************************************************************************
 */

public class SQLLogUtil
{
	private static StringComparator	stringComparator	= new StringComparator(false);

	public static void logBeforeExecutingQuery(String sql, Log log)
	{
		if (log.isDebugEnabled())
			log.debug("SQL [" + sql + "]");
	}

	public static void logBeforeExecutingBatch(String sql, Log log)
	{
		if (log.isDebugEnabled())
			log.debug("SQL Batch [" + sql + "]");
	}
	
	public static void logBeforeExecutingProcedure( String name, Map args, Log log )
	{
		if (log.isDebugEnabled())
			log.debug( "PROCEDURE [" + name + "], IN Parameters [" + getLogFormatParameter( args ) + "]" );
	}
	
	public static void logAfterExecutingQuery(Object result, Log log)
	{
		if (log.isDebugEnabled())
		{
			if (result instanceof List)
				log.debug("SQL query returned " + ((List) result).size() + " rows");
			else if (result instanceof DataInfo)
				log.debug("SQL query returned 1 row");
			else
				log.debug("SQL query returned " + (Integer) result);
		}
	}
	
	public static void logAfterExecutingBatch(int[] result, Log log)
	{
		if (log.isDebugEnabled())
			log.debug( "Batch Size=" + result.length);
	}

	public static void logAfterExecutingUpdate(int result, Log log)
	{
		if (log.isDebugEnabled())
			log.debug("SQL update affected " + result + " rows");
	}
	
	public static void logAfterExecutingProcedure( String name, Map result, Log log )
	{
		if (log.isDebugEnabled())
			log.debug( "PROCEDURE [" + name + "], OUT Parameters [" + getLogFormatParameter( result ) + "]" );
	}

	public static String getLogFormatSqlStatement(String sql, Object args, Log log)
	{
		try
		{
			if (args instanceof Map)
			{
				Map<String, Object> map = (Map<String, Object>) args;

				TreeMap<String, Object> treeMap = new TreeMap<String, Object>(stringComparator);
				treeMap.putAll(map);

				Iterator<Entry<String, Object>> iter = treeMap.entrySet().iterator();
				while (iter.hasNext())
				{
					Entry<String, Object> entry = iter.next();
					sql = sql.replace(":" + entry.getKey(), getLogFormatArgument(entry.getValue()));
				}
			}
			else if (args instanceof Object[])
			{
				Object[] objs = (Object[]) args;

				for (Object obj : objs)
				{
					sql = StringUtils.replaceOnce(sql, "?", getLogFormatArgument(obj));
				}
			}
		} catch (Throwable t)
		{
			if (log.isDebugEnabled())
				log.error(t, t);
			else
				log.error(t);
		}

		return sql;
	}

	public static String getLogFormatArgument(Object arg)
	{
		if (arg instanceof String)
			return "'" + (String)arg + "'";
		else if (arg instanceof Timestamp || arg instanceof Date)
			return "timestamp '" + arg.toString() + "'";
		else if ( arg == null )
			return "null";
		else
			return arg.toString();
	}
	
	public static String getLogFormatParameter( Map args )
	{
		String result = null;
		StringBuilder builder = new StringBuilder();
		
		if ( args == null )
			result = "";
		else
		{
			Iterator iter = args.entrySet().iterator();
			
			while ( iter.hasNext())
			{
				Entry entry = ( Entry )iter.next();
				builder.append( entry.getKey()).append( "=" ).append( getLogFormatArgument( entry.getValue()));
				builder.append( ", " );
			}
			
			if ( builder.length() > 0 )
				result = builder.substring( 0, builder.length() - 2 );
			else
				result = builder.toString();
		}
		
		return result;
	}
}

class StringComparator implements Comparator<String>
{
	private boolean	sortOrderAscending	= true;

	public StringComparator()
	{
	}

	public StringComparator(boolean sortOrderAscending)
	{
		this.sortOrderAscending = sortOrderAscending;
	}

	public int compare(String o1, String o2)
	{
		int result = o1.compareTo(o2);
		return sortOrderAscending ? result : -result;
	}
}