package com.cim.idm.framework.orm;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;


import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.database.JdbcCursorItemReader;
import org.springframework.jdbc.core.ColumnMapRowMapper;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.util.LinkedCaseInsensitiveMap;

import com.cim.idm.framework.util.sys.SystemPropHelper;

/*
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.orm
 *  NAME    : SqlCursorItemReader.java
 *  TYPE    : JAVA
 *  DESCRIPTION :
 *
 ****************************************************************************
 */

public class SqlCursorItemReader
{
	private JdbcCursorItemReader	reader;
	private String					appendSpace	= "";

	public void setAppendSpace(String appendSpace)
	{
		this.appendSpace = appendSpace;
	}

	public SqlCursorItemReader(JdbcCursorItemReader jdbcCursorItemReader)
	{
		this.reader = jdbcCursorItemReader;
	}

	public void setRowMapper(RowMapper rowMapper)
	{
		this.reader.setRowMapper(rowMapper);
	}

	public void setColumnMapRowMapper()
	{
		reader.setRowMapper(new ColumnMapRowMapper());
	}

	public void setListMapper()
	{
		createListMapper();
	}

	public void setListMapper(List<List<ObjectAttributeDef>> objectAttributeDefs)
	{
		createListMapper(objectAttributeDefs);
	}

	public void setXmlStringMapper()
	{
		createXmlStringMapper();
	}

	public void open()
	{
		ExecutionContext context = new ExecutionContext();
		reader.open(context);
	}

	public Object read()
	{
		try
		{
			return this.reader.read();
		} catch (Exception e)
		{
			return null;
		}
	}

	private void createListMapper()
	{
		RowMapper mapper = new RowMapper() {
			public Object mapRow(ResultSet rs, int i) throws SQLException
			{
				LinkedCaseInsensitiveMap<Object> item = new LinkedCaseInsensitiveMap<>();
				for (int j = 1; j <= rs.getMetaData().getColumnCount(); j++)
				{
					String columnName = rs.getMetaData().getColumnName(j);
					item.put(columnName, rs.getObject(columnName));
				}
				return item;
			}
		};
		reader.setRowMapper(mapper);
	}

	private void createListMapper(final List<List<ObjectAttributeDef>> objectAttributeDefs)
	{
		RowMapper mapper = new RowMapper() {
			public Object mapRow(ResultSet rs, int i) throws SQLException
			{
				LinkedCaseInsensitiveMap<Object> item = new LinkedCaseInsensitiveMap<>();
				for (int h = 0; h < 2; h++)
				{
					List<ObjectAttributeDef> ObjectAttributeDefList =
							(List<ObjectAttributeDef>) objectAttributeDefs.get(h);
					if (ObjectAttributeDefList == null)
						continue;
					for (int j = 0; j < ObjectAttributeDefList.size(); j++)
					{
						item.put(ObjectAttributeDefList.get(j).getAttributeName(),
							rs.getObject(ObjectAttributeDefList.get(j).getAttributeName()));
					}
				}
				return item;
			}
		};
		reader.setRowMapper(mapper);
	}

	private void createXmlStringMapper()
	{
		RowMapper mapper = new RowMapper() {
			public Object mapRow(ResultSet rs, int i) throws SQLException
			{
				StringBuffer strBuffer = new StringBuffer();
				for (int j = 1; j <= rs.getMetaData().getColumnCount(); j++)
				{
					String columnName = rs.getMetaData().getColumnName(j);
					strBuffer.append(appendSpace).append("<").append(columnName).append(">");
					strBuffer.append(rs.getObject(columnName)).append("</").append(columnName).append(">").append(
						SystemPropHelper.CR);
				}
				return strBuffer.toString();
			}
		};
		reader.setRowMapper(mapper);
	}

	public void close()
	{
		this.reader.close();
	}

}
