package com.cim.idm.framework.orm;

import java.math.BigDecimal;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.batch.item.database.JdbcCursorItemReader;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.ColumnMapRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.SingleColumnRowMapper;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcOperations;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcCall;
import org.springframework.util.ObjectUtils;

import com.cim.idm.framework.exception.ErrorSignal;
import com.cim.idm.framework.exception.IDMFrameDBErrorSignal;
import com.cim.idm.framework.util.object.ObjectUtil;
import com.cim.idm.framework.util.sys.SystemPropHelper;

public class SqlTemplate implements InitializingBean
{
	private static Log			log				= LogFactory.getLog(SqlTemplate.class);

	private DataSource			dataSource;

	private JdbcTemplate		jdbcTemplate;
	
	private NamedParameterJdbcOperations namedParameterJdbcOperations;
	
	private RowMapper<Map<String,Object>>	rowMapper = new ColumnMapRowMapper();

	private int					queryTimeout	= 0;
	
	public static  String		RDB_TYPE	= "ORACLE";
	
	public static  String 		jdbcDriverName = "";
	
	private int 				batchSize = 2000; // max Batch process size for each batch update
	
	public SqlTemplate() {
	}
	
	public DataSource getDataSource()
	{
		return this.dataSource;
	}

	public void setDataSource(DataSource dataSource)
	{
		this.dataSource = dataSource;
	}

	public int getQueryTimeout()
	{
		return queryTimeout;
	}

	public void setQueryTimeout(int queryTimeout)
	{
		this.queryTimeout = queryTimeout;
	}

	public JdbcTemplate getJdbcTemplate()
	{
		return this.jdbcTemplate;
	}
	
	/**
	 * Expose the Spring NamedParameterJdbcTemplate to allow invocation of
	 * less commonly used methods.
	 */
	public NamedParameterJdbcOperations getNamedParameterJdbcOperations() {
		return this.namedParameterJdbcOperations;
	}

	public String getRDBType()
	{
		return this.RDB_TYPE;
	}
	
	public String getJdbcDriverName() {
		return jdbcDriverName;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate)
	{
		this.jdbcTemplate = jdbcTemplate;
	}

	public void setRowMapper( RowMapper<Map<String,Object>> rowMapper )
	{
		this.rowMapper = rowMapper;
	}
	
	public int getBatchSize() {
		return batchSize;
	}

	public void setBatchSize(int batchSize) {
		this.batchSize = batchSize;
	}

	@Override
	public void afterPropertiesSet() throws Exception
	{
		if (this.dataSource == null)
		{
			if (this.jdbcTemplate != null)
			{
				this.dataSource = this.jdbcTemplate.getDataSource();

				if (this.queryTimeout <= 0)
					this.queryTimeout = this.jdbcTemplate.getQueryTimeout();
			}
		}

		if (this.jdbcTemplate == null)
		{
			this.jdbcTemplate = new JdbcTemplate(this.dataSource);
			this.jdbcTemplate.setQueryTimeout(this.queryTimeout);
		}
		this.namedParameterJdbcOperations = new NamedParameterJdbcTemplate(this.jdbcTemplate);
		
		RDB_TYPE = dataSource.getConnection().getMetaData().getDatabaseProductName();
		jdbcDriverName = dataSource.getConnection().getMetaData().getDriverName();
		

	}

	public SqlCursorItemReader queryByCursor(String sql, final Object... args)
	{
		JdbcCursorItemReader reader = new JdbcCursorItemReader();
		reader.setDataSource(getJdbcTemplate().getDataSource());
		reader.setSql(sql);

		if (args != null)
		{
			PreparedStatementSetter preparedStatementSetter = new PreparedStatementSetter() {
				public void setValues(PreparedStatement preparedStatement) throws SQLException
				{
					for (int i = 0; i < args.length; i++)
					{
						preparedStatement.setObject(i + 1, args[i]);
					}
				}
			};
			reader.setPreparedStatementSetter(preparedStatementSetter);
		}

		return new SqlCursorItemReader(reader);
	}

	public int[] updateBatch(String... querys) throws IDMFrameDBErrorSignal
	{
		try
		{
			int[] result = null;
			
			SQLLogUtil.logBeforeExecutingBatch( "", log );
			
			result = getJdbcTemplate().batchUpdate(querys);
			
			SQLLogUtil.logAfterExecutingBatch( result, log );
			
			return result;
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e);
		}
	}

	public int[] updateBatch(String query, BatchPreparedStatementSetter pss) throws IDMFrameDBErrorSignal
	{
		try
		{
			int[] result = null;
			
			SQLLogUtil.logBeforeExecutingBatch( query, log );
			
			result = getJdbcTemplate().batchUpdate(query, pss);
			
			SQLLogUtil.logAfterExecutingBatch( result, log );
			
			return result;
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, query);
		}
	}
	/**
	 * Executes a batch using the supplied SQL statement with the batch of supplied arguments.
	 * Uses sql with the named parameter support.<br/>
	 * If the length of batchValues is more than <em>batchSize</em>, separate the array with <em>batchSize</em>.
	 * @param sql the SQL statement to execute.
	 * @param batchValues the array of Maps containing the batch of arguments for the query.
	 * @return an array containing the numbers of rows affected by each update in the batch.
	 */
	public int[] updateBatch(String sql, Map<String,Object>[] batchValues) throws IDMFrameDBErrorSignal
	{
		try
		{
			int[] result = null;
			int[] subResult = null;
			Map<String,Object>[] subArgs = null;
			
			SQLLogUtil.logBeforeExecutingBatch( sql, log );
			int t = batchValues.length / batchSize;
			if(t==0)
				result = getNamedParameterJdbcOperations().batchUpdate(sql, batchValues);
			else{
				result = new int[batchValues.length];
				for(int i = 0 ; i < t; i++ ){
					subArgs = new Map[batchSize];
					for(int j = 0 ; j < batchSize; j++)
						subArgs[j] = batchValues[i*batchSize+j];

					subResult = getNamedParameterJdbcOperations().batchUpdate(sql, subArgs);
					System.arraycopy(subResult, 0, result, i*batchSize, batchSize);
				}
				subArgs = new Map[batchValues.length-t*batchSize];
				for(int j = 0 ; j < subArgs.length; j++)
					subArgs[j] = batchValues[t*batchSize+j];
				subResult = getNamedParameterJdbcOperations().batchUpdate(sql, subArgs);
				System.arraycopy(subResult, 0, result, t*batchSize, subArgs.length);
			}
			SQLLogUtil.logAfterExecutingBatch( result, log );
			
			return result;
			
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, sql);
		}
	}
	/**
	 * Execute a batch using the supplied SQL statement with the batch of supplied arguments.
	 * Uses sql with the standard '?' placeholders for parameters<br/>
	 * If the size of batchValues is more than <em>batchSize</em>, separate the list with <em>batchSize</em>.
	 * @param sql the SQL statement to execute.
	 * @param batchArgs the List of Object arrays containing the batch of arguments for the query.
	 * @return an array containing the numbers of rows affected by each update in the batch.
	 */
	public int[] updateBatch(String sql, List<Object[]> batchArgs) throws IDMFrameDBErrorSignal
	{
		try
		{
			int[] result = null;
			List<Object[]> subList = null;
			int[] subResult = null;
			SQLLogUtil.logBeforeExecutingBatch( sql, log );
			int t = batchArgs.size() / batchSize;
			if(t==0)
				result = getJdbcTemplate().batchUpdate(sql, batchArgs);
			else{
				result = new int[batchArgs.size()];
				for(int i = 0 ; i < t; i++ ){
					subList = batchArgs.subList(i*batchSize, (i+1)*batchSize);
					subResult = getJdbcTemplate().batchUpdate(sql, subList);
					System.arraycopy(subResult, 0, result, i*batchSize, batchSize);
				}
				subList = batchArgs.subList(t*batchSize, batchArgs.size());
				subResult = getJdbcTemplate().batchUpdate(sql, subList);
				System.arraycopy(subResult, 0, result, t*batchSize, subList.size());
			}
			SQLLogUtil.logAfterExecutingBatch( result, log );
			
			return result;
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, sql);
		}
	}

	public int[] updateBatchByList(String query, final List<List<Object>> params) throws IDMFrameDBErrorSignal
	{
		try
		{
			return updateBatch(query, new BatchPreparedStatementSetter() {
				public int getBatchSize()
				{
					return params.size();
				}

				public void setValues(PreparedStatement ps, int i) throws SQLException
				{
					List<Object> param = params.get(i);
					for (int j = 0; j < (null != param ? param.size() : 0); j++)
					{
						Object obj = param.get(j);
						if (obj instanceof BigDecimal)
						{
							if (!obj.toString().contains("."))
								ps.setObject(j + 1, Long.valueOf(obj.toString()));
							else
								ps.setObject(j + 1, Double.valueOf(obj.toString()));

						}
						else
							ps.setObject(j + 1, obj);
					}
				}
			});
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, query);
		}
	}

	public int[] updateBatch(String query, SqlParameterSource[] args) throws IDMFrameDBErrorSignal
	{
		try
		{
			int[] result = null;
			
			SQLLogUtil.logBeforeExecutingBatch( query, log );
			
			result = getNamedParameterJdbcOperations().batchUpdate(query, args);
			
			SQLLogUtil.logAfterExecutingBatch( result, log );
			
			return result;
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, query);
		}
	}

	public Map<String,Object> executeProcedure( String procedureName ) throws IDMFrameDBErrorSignal
	{
		return executeProcedure( procedureName, null );
	}

	public Map<String,Object> executeProcedure( String procedureName, Map<String,?> args ) throws IDMFrameDBErrorSignal
	{
		return executeProcedure( procedureName, args, new SqlParameter[0]);
	}

	public Map<String,Object> executeProcedure( String procedureName, Map<String,?> args, SqlParameter... declareSqlParameters ) throws IDMFrameDBErrorSignal
	{
		try
		{
			SQLLogUtil.logBeforeExecutingProcedure( procedureName, args, log );
			
			SimpleJdbcCall procReader = new SimpleJdbcCall( jdbcTemplate );

			if ( !StringUtils.contains( procedureName, '.' ) )
				procReader = procReader.withProcedureName( procedureName );
			else
			{
				String[] names = StringUtils.split( procedureName, '.' );
				procReader = procReader.withCatalogName( names[0] ).withProcedureName( names[1] );
			}
			
			if ( declareSqlParameters != null && declareSqlParameters.length > 0 )
				procReader.withoutProcedureColumnMetaDataAccess().declareParameters( declareSqlParameters );
			
			Map<String,Object> out = null;
			if ( MapUtils.isEmpty( args ))
				out = procReader.execute();
			else
				out = procReader.execute( new MapSqlParameterSource().addValues( args ));
			
			SQLLogUtil.logAfterExecutingProcedure( procedureName, out, log );
			
			return out;
		} 
		catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), procedureName);
		}
	}

	public String[][] queryForStringArray(String sql, Object... args) throws IDMFrameDBErrorSignal
	{
		return DataAccessHandler.getStringArrayData(queryForList(sql, args));
	}

	public int queryForInt(String sql, Object... args) throws IDMFrameDBErrorSignal
	{
		int result = 0;
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if (ArrayUtils.getLength( args )== 0)
				result = getJdbcTemplate().queryForObject(sql,Integer.class);
			else
				result = getJdbcTemplate().queryForObject(sql, args,Integer.class);
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingQuery(result, log);
		return result;
	}

	public int queryForInt(String sql, Map<String, ?> args) throws IDMFrameDBErrorSignal
	{
		int result = 0;
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if (args == null)
				result = this.getJdbcTemplate().queryForObject(sql,Integer.class);
			else {
				result = getNamedParameterJdbcOperations().queryForObject(sql, args, Integer.class);
			}

		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingQuery(result, log);
		return result;
	}

	/**
	 * 
	 * @return List<Map<String, Object>>, List<LinkedCaseInsensitiveMap>
	 * @throws IDMFrameDBErrorSignal
	 */
	public List<Map<String,Object>> queryForList(String sql, Object... args) throws IDMFrameDBErrorSignal
	{
		List<Map<String,Object>> resultList = null;
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if (ArrayUtils.isEmpty(args))
				resultList = getJdbcTemplate().query(sql, this.rowMapper );
			else
			{
				if (args.length == 1 && args[0] == null)
					resultList = getJdbcTemplate().query(sql, this.rowMapper );
				else
					resultList = getJdbcTemplate().query(sql, args, this.rowMapper );
			}
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingQuery(resultList, log);
		return resultList;
	}

	/**
	 * 
	 * @return List<Map<String, Object>>, List<LinkedCaseInsensitiveMap>
	 * @throws IDMFrameDBErrorSignal
	 */
	public List<Map<String,Object>> queryForList(String sql, Map<String,?> args) throws IDMFrameDBErrorSignal
	{
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);
		List<Map<String,Object>> resultList = null;

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if ( this.rowMapper instanceof NullReplaceableRowMapper || this.rowMapper instanceof ColumnMapRowMapper )
			{
				if (args == null)
					resultList = this.getJdbcTemplate().query(sql, (ColumnMapRowMapper)this.rowMapper);
				else
					resultList = getNamedParameterJdbcOperations().query(sql, args ,(ColumnMapRowMapper)this.rowMapper);
			}
			else
			{
				if (args == null)
					resultList = this.getJdbcTemplate().queryForList( sql );
				else
					resultList = getNamedParameterJdbcOperations().queryForList(sql, args);
			}
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingQuery(resultList, log);
		return resultList;
	}

	public <T> T queryForObject(String sql, Class<T> requiredType, Map<String,?> args) throws IDMFrameDBErrorSignal
	{
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);
			
			SingleColumnRowMapper<T> singleRowMapper = null;
			
			if ( rowMapper instanceof NullReplaceableSingleRowMapper )
				singleRowMapper = new NullReplaceableSingleRowMapper<>(requiredType);
			else
				singleRowMapper = new SingleColumnRowMapper<>(requiredType);
			
			if ( singleRowMapper instanceof NullReplaceableSingleRowMapper )
				return getNamedParameterJdbcOperations().queryForObject(sql, args, (RowMapper<T>)singleRowMapper);
			else
				return getNamedParameterJdbcOperations().queryForObject(sql, args, requiredType);
			
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}
	}

	public <T> T queryForObject(String sql, Class<T> requiredType, Object... args) throws IDMFrameDBErrorSignal
	{
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);
			
			SingleColumnRowMapper<T> singleRowMapper = null;
			
			if ( rowMapper instanceof NullReplaceableSingleRowMapper )
				singleRowMapper = new NullReplaceableSingleRowMapper<>(requiredType);
			else
				singleRowMapper = new SingleColumnRowMapper<>(requiredType);
			
			if ( singleRowMapper instanceof NullReplaceableSingleRowMapper )
				return this.getJdbcTemplate().queryForObject(sql, (RowMapper<T>)singleRowMapper, args);
			else
				return this.getJdbcTemplate().queryForObject(sql, requiredType, args);
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}
	}

	public Map<String, Object> queryForMap(String sql, Map<String,?> args) throws DataAccessException {
		return getNamedParameterJdbcOperations().queryForMap(sql, args);
	}
	
	public Map<String, Object> queryForMap(String sql, Object... args) throws DataAccessException {
		return (ObjectUtils.isEmpty(args) ?
				getNamedParameterJdbcOperations().getJdbcOperations().queryForMap(sql) :
				getNamedParameterJdbcOperations().getJdbcOperations().queryForMap(sql, args));
	}
	/**
	 * 
	 * @see Reserved words : STARTROW, ENDROW, ROWSEQ ==> args Key Name : STARTROW, ENDROW, return vlaue : ROWSEQ
	 * 
	 * @return List<Map<String, Object>>, List<LinkedCaseInsensitiveMap>
	 */
	public List<Map<String,Object>> queryForListByPaging(int pageIndex, int pageSize, String sql, Map<String, ?> args)
			throws IDMFrameDBErrorSignal
	{
		String newSql =
				"SELECT * FROM ( SELECT data.*, rownum as ROWSEQ FROM "
					+ SystemPropHelper.CR
					+ "  ("
					+ SystemPropHelper.CR
					+ sql
					+ SystemPropHelper.CR
					+ "  ) data WHERE rownum <= :ENDROW "
					+ SystemPropHelper.CR
					+ ") WHERE ROWSEQ BETWEEN  :STARTROW AND :ENDROW";
		Map<String, Object> newArgs = null;
		int startRow = (pageIndex - 1) * pageSize + 1;
		int endRow = pageIndex * pageSize;

		newArgs = new HashMap<String, Object>(args);
		newArgs.put("STARTROW", startRow);
		newArgs.put("ENDROW", endRow);

		return queryForList(newSql, newArgs);
	}

	/**
	 * 
	 * @return List<Map<String, Object>>, List<LinkedCaseInsensitiveMap>
	 * @throws IDMFrameDBErrorSignal
	 */
	public List<Map<String,Object>> queryForListByPaging(int pageIndex, int pageSize, String sql, Object... args)
			throws IDMFrameDBErrorSignal
	{
		String newSql =
				"SELECT * FROM ( SELECT data.*, rownum as ROWSEQ FROM "
					+ SystemPropHelper.CR
					+ "  ("
					+ SystemPropHelper.CR
					+ sql
					+ SystemPropHelper.CR
					+ "  ) data WHERE rownum <= ? "
					+ SystemPropHelper.CR
					+ ") WHERE ROWSEQ BETWEEN ? AND ?";
		Object[] newArgs = null;

		int startRow = (pageIndex - 1) * pageSize + 1;
		int endRow = pageIndex * pageSize;

		int cnt = args != null ? args.length : 0;
		newArgs = new Object[cnt + 3];
		for (int index = 0; index < cnt; index++)
		{
			newArgs[index] = args[index];
		}

		newArgs[cnt] = endRow;
		newArgs[cnt + 1] = startRow;
		newArgs[cnt + 2] = endRow;

		return queryForList(newSql, newArgs);
	}

	public int update(String sql, Object... args) throws IDMFrameDBErrorSignal
	{
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);
		int result = 0;

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if ( ArrayUtils.getLength( args ) == 0 )
				result = getJdbcTemplate().update(sql);
			else
				result = getJdbcTemplate().update(sql, args);
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingUpdate(result, log);
		return result;
	}

	public int update(String sql, Map<String, ?> args) throws IDMFrameDBErrorSignal
	{
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);

		int result = 0;
		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if (args == null)
				result = this.getJdbcTemplate().update(sql);
			else
				result = getNamedParameterJdbcOperations().update(sql, args);
		} catch (DataAccessException e)
		{
			throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingUpdate(result, log);
		return result;
	}

	public int update(String sql, Object[] args, String keyString) throws IDMFrameDBErrorSignal
	{
		String logFormatSql = SQLLogUtil.getLogFormatSqlStatement(sql, args, log);
		int result = 0;

		try
		{
			SQLLogUtil.logBeforeExecutingQuery(logFormatSql, log);

			if (args == null)
				result = getJdbcTemplate().update(sql);
			else
				result = getJdbcTemplate().update(sql, args);
		} 
		catch (DataAccessException e)
		{
			if (keyString != null)
				throw ErrorSignal.getNotifyException(e, keyString, logFormatSql);
			else
				throw ErrorSignal.getNotifyException(e, ObjectUtil.getString(args), logFormatSql);
		}

		SQLLogUtil.logAfterExecutingUpdate(result, log);
		return result;
	}

}
