package com.cim.idm.framework.orm;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;

public class Update {
	
	private String tableName;
	private String versionColumnName;
	private String where;
	private String assignments;
	private String comment;
	private List<LinkedHashMap<String, String>> ORConditionMapList = new ArrayList<LinkedHashMap<String,String>>();
	private Map<String,String> primaryKeyColumns = new LinkedHashMap<>();
	private Map<String,String> columns = new LinkedHashMap<>();//update columns
	private Map<String,String> whereColumns = new LinkedHashMap<>();//where condition columns if no "or" express conditon 
	
	public Update() {
		
	}
		
	public Update setTableName(String tableName) {
		this.tableName = tableName;
		return this;
	}
	
	public Update setORConditionMapList(List<List<String>> ORConditionColumnNames){
		this.ORConditionMapList.clear();
		for(List<String> ColumnNames : ORConditionColumnNames){
			LinkedHashMap<String, String> ORConditionMap = new LinkedHashMap<String,String>();
			for(String ColumnName : ColumnNames){
				ORConditionMap.put(ColumnName, "=?");
			}
			ORConditionMapList.add(ORConditionMap);	
		}
		return this;
	}

	public Update setPrimaryKeyColumnNames(String[] columnNames) {
		this.primaryKeyColumns.clear();
		addPrimaryKeyColumns(columnNames);
		return this;
	}	
	
	public Update addPrimaryKeyColumns(String[] columnNames) {
		for ( int i=0; i<columnNames.length; i++ ) {
			addPrimaryKeyColumn( columnNames[i], "?" );
		}
		return this;
	}
	
	public Update addPrimaryKeyColumn(String columnName, String valueExpression) {
		this.primaryKeyColumns.put(columnName, valueExpression);
		return this;
	}
	
	public Update addPrimaryKeyColumns(String[] columnNames, String[] valueExpressions) {
		for ( int i=0; i<columnNames.length; i++ ) {
			addPrimaryKeyColumn( columnNames[i], valueExpressions[i] );
		}
		return this;
	}	
	
	public Update addColumns(String[] columnNames) {
		for ( int i=0; i<columnNames.length; i++ ) {
			addColumn( columnNames[i] );
		}
		return this;
	}
	
	public Update addColumn(String columnName) {
		return addColumn(columnName, "?");
	}
	
	public Update addColumn(String columnName, String valueExpression) {
		columns.put(columnName, valueExpression);
		return this;
	}
	
	public Update addColumns(String[] columnNames, String valueExpression) {
		for ( int i=0; i<columnNames.length; i++ ) {
			addColumn( columnNames[i], valueExpression );
		}
		return this;
	}
	
	public Update addWhereColumns(String[] columnNames) {
		for ( int i=0; i<columnNames.length; i++ ) {
			addWhereColumn( columnNames[i] );
		}
		return this;
	}

	public Update addWhereColumns(String[] columnNames, String valueExpression) {
		for ( int i=0; i<columnNames.length; i++ ) {
			addWhereColumn( columnNames[i], valueExpression );
		}
		return this;
	}

	public Update addWhereColumn(String columnName) {
		return addWhereColumn(columnName, "=?");
	}

	public Update addWhereColumn(String columnName, String valueExpression) {
		if(whereColumns.containsKey(columnName)){
			whereColumns.put(columnName, whereColumns.get(columnName)+","+valueExpression);
		}else{
			whereColumns.put(columnName, valueExpression);
		}
		return this;
	}

	public Update setWhere(String where) {
		this.where=where;
		return this;
	}

	public String toStatementString() {
		StringBuilder buf = new StringBuilder( (columns.size() * 15) + tableName.length() + 10 );
		if ( comment!=null ) {
			buf.append( "/* " ).append( comment ).append( " */ " );
		}
		buf.append( "update " ).append( tableName ).append( " set " );
		boolean assignmentsAppended = false;
		Iterator<Entry<String, String>> iter = columns.entrySet().iterator();
		while ( iter.hasNext() ) {
			Entry<String, String> e =  iter.next();
			buf.append( e.getKey().toUpperCase()).append( '=' ).append( e.getValue() );
			if ( iter.hasNext() ) {
				buf.append( ", " );
			}
			assignmentsAppended = true;
		}
		if ( assignments != null ) {
			if ( assignmentsAppended ) {
				buf.append( ", " );
			}
			buf.append( assignments );
		}

		boolean conditionsAppended = false;
		if ( !ORConditionMapList.isEmpty() ||!primaryKeyColumns.isEmpty() || where != null || !whereColumns.isEmpty() || versionColumnName != null ) {
			buf.append( " where " );
		}
		if(!ORConditionMapList.isEmpty()){
			buf.append("(");
			if(ORConditionMapList.size()>1)
			buf.append("(");
			ListIterator<LinkedHashMap<String,String>> listIter = ORConditionMapList.listIterator();
			while(listIter.hasNext()){
				
				Map<String, String> ORConditionMap = listIter.next();
				
				iter = ORConditionMap.entrySet().iterator();
				while(iter.hasNext()){
					Entry<String, String> e = iter.next();
					buf.append( e.getKey().toUpperCase() ).append( e.getValue() );
					if ( iter.hasNext() ) {
						buf.append( " and " );
					}
				}
	
				if(listIter.hasNext())
					buf.append(") or (");
				else
					if(ORConditionMapList.size()>1)
						buf.append(")");
			}
			buf.append(")");
			conditionsAppended = true;
		}
		
		if(!primaryKeyColumns.isEmpty()){
			if ( conditionsAppended ) {
				buf.append( " and " );
			}
			iter = primaryKeyColumns.entrySet().iterator();
			while ( iter.hasNext() ) {
				Entry<String, String> e =  iter.next();
				buf.append( e.getKey().toUpperCase() ).append( '=' ).append( e.getValue() );
				if ( iter.hasNext() ) {
					buf.append( " and " );
				}
			}
			conditionsAppended = true;
		}
		if ( where != null ) {
			if ( conditionsAppended ) {
				buf.append( " and " );
			}
			buf.append( where );
			conditionsAppended = true;
		}
		iter = whereColumns.entrySet().iterator();
		while ( iter.hasNext() ) {
			final Entry<String, String> e =  iter.next();
			if ( conditionsAppended ) {
				buf.append( " and " );
			}
			if(e.getValue().toString().contains(",")){
				buf.append( e.getKey().toUpperCase() ).append(" in ").append("( ").append( StringUtils.remove(e.getValue().toString(), "=") ).append(" )");
			}else{
				buf.append( e.getKey().toUpperCase() ).append( e.getValue() );
			}
			conditionsAppended = true;
		}
		if ( versionColumnName != null ) {
			if ( conditionsAppended ) {
				buf.append( " and " );
			}
			buf.append( versionColumnName ).append( "=?" );
		}

		return buf.toString();
	}
}
