package com.cim.idm.framework.orm.service;

import java.sql.Timestamp;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;

import com.cim.idm.framework.IDMFrameServiceProxy;
import com.cim.idm.framework.data.TransitionInfo;
import com.cim.idm.framework.exception.ext.DuplicateNameSignal;
import com.cim.idm.framework.exception.ext.ExceptionNotify;
import com.cim.idm.framework.exception.ext.FrameworkErrorSignal;
import com.cim.idm.framework.exception.ext.NotFoundSignal;
import com.cim.idm.framework.orm.info.DataInfo;
import com.cim.idm.framework.orm.info.HistoryDataInfo;
import com.cim.idm.framework.orm.info.KeyInfo;
import com.cim.idm.framework.orm.support.OrmStandardEngineUtil;
import com.cim.idm.framework.util.object.ObjectUtil;
import com.cim.idm.framework.util.service.ServiceUtil;
import com.cim.idm.framework.util.time.TimeStampUtil;
import com.cim.idm.framework.util.time.TimeUtils;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.orm.service
 *  NAME    : CommonDataServiceDAO.java
 *  TYPE    : JAVA
 *  DESCRIPTION : An extension of the Common service
 *
 ****************************************************************************
 */

public class CommonDataServiceDAO<KEY extends KeyInfo, DATA extends DataInfo> extends CommonServiceDAO<KEY, DATA>
		implements CommonDataService<KEY, DATA>
{
	@Autowired
	private ApplicationContext applicationContext;

	public CommonDataServiceDAO()
	{
	}

	public void create(DATA data, String user, String comment)
			throws NotFoundSignal, DuplicateNameSignal, FrameworkErrorSignal
	{
		IDMFrameServiceProxy.getTxDataSourceManager().beginTransaction();
		try
		{
			this.insert(data);

			addHistory(data, TimeStampUtil.getCurrentTimestamp(), "create", user, comment);
			IDMFrameServiceProxy.getTxDataSourceManager().commitTransaction();
		} catch (Throwable e)
		{
			IDMFrameServiceProxy.getTxDataSourceManager().rollbackTransaction();
			ExceptionNotify.notifyException(e);
		}

	}

	public void changeData(DATA data, String user, String comment) throws NotFoundSignal, FrameworkErrorSignal
	{
		IDMFrameServiceProxy.getTxDataSourceManager().beginTransaction();
		try
		{
			this.selectByKeyForUpdate((KEY) data.getKey());

			this.update(data);
			addHistory(data, TimeUtils.getCurrentTimestamp(), "changeData", user, comment);

			IDMFrameServiceProxy.getTxDataSourceManager().commitTransaction();
		} catch (Throwable e)
		{
			IDMFrameServiceProxy.getTxDataSourceManager().rollbackTransaction();
			ExceptionNotify.notifyException(e);
		}
	}

	protected void addHistory(DATA data, Timestamp eventTime, String eventName, String user, String comment)
			throws FrameworkErrorSignal, DuplicateNameSignal
	{
		IDMFrameServiceProxy.getTxDataSourceManager().beginTransaction();
		try
		{
			// DataHistory creating and insert.
			String historyServiceClass = OrmStandardEngineUtil.getHistoryPackageName(data);
			HistoryDataInfo historyData = (HistoryDataInfo) ObjectUtil.newInstanceByClassName(data.getClass().getName() + "History");

			CommonHistoryService historyService =
					(CommonHistoryService) ServiceUtil.getServiceByClassName(applicationContext,historyServiceClass);

			ObjectUtil.copyFieldNotEqual(OrmStandardEngineUtil.getKeyInfo(data), historyData.getKey());
			ObjectUtil.copyFieldNotEqual(data, historyData);

			ObjectUtil.setFieldValue(historyData.getKey(), "timeKey", TimeUtils.getEventTimeKeyFromTimestamp(eventTime));
			ObjectUtil.setFieldValue(historyData, "eventTime", eventTime);
			ObjectUtil.setFieldValue(historyData, "eventName", eventName);
			ObjectUtil.setFieldValue(historyData, "eventUser", user);
			ObjectUtil.setFieldValue(historyData, "eventComment", comment);
			historyService.insert(historyData);

			IDMFrameServiceProxy.getTxDataSourceManager().commitTransaction();
		} catch (Throwable e)
		{
			IDMFrameServiceProxy.getTxDataSourceManager().rollbackTransaction();
			ExceptionNotify.notifyException(e);
		}
	}

}
