package com.cim.idm.framework.orm.service;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cim.idm.framework.exception.ext.ExceptionKey;
import com.cim.idm.framework.exception.ext.FrameworkErrorSignal;
import com.cim.idm.framework.orm.info.CRData;
import com.cim.idm.framework.orm.info.DataInfo;
import com.cim.idm.framework.orm.info.HistoryDataInfo;
import com.cim.idm.framework.orm.info.HistoryKeyInfo;
import com.cim.idm.framework.orm.support.OrmStandardEngineUtil;
import com.cim.idm.framework.util.object.ObjectUtil;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.orm.service
 *  NAME    : CommonHistoryService.java
 *  TYPE    : JAVA
 *  DESCRIPTION : Adaptor for Common History DB access
 *
 ****************************************************************************
 */

public abstract class CommonHistoryDataAdaptor<DATA extends DataInfo, HKEY extends HistoryKeyInfo, HDATA extends HistoryDataInfo>
{
	protected Log	log	= LogFactory.getLog(this.getClass());

	protected abstract HKEY getHistoryKey(DATA dataInfo);

	/**
	 * 
	 * @param data
	 *            extends com.cim.idm.framework.orm.info.DataInfo
	 * @param crData
	 *            com.cim.idm.framework.generic.info.CRData
	 * @return DATA extends com.cim.idm.framework.orm.info.DataInfo
	 * @throws FrameworkErrorSignal
	 * @test
	 * @code
	 * @endcode
	 */
	public DATA replace(DATA data, CRData<HDATA> crData) throws FrameworkErrorSignal
	{
		if (crData.getRecoveredData() == null)
		{
			HDATA newData = (HDATA) OrmStandardEngineUtil.createDataInfo(crData.getCanceledData());
			crData.setRecoveredData(newData);
		}

		recover(data, crData.getRecoveredData());

		return data;
	}

	/**
	 * 
	 * @param data
	 *            extends com.cim.idm.framework.orm.info.DataInfo
	 * @param historyData
	 *            extends com.cim.idm.framework.orm.info.DataInfo
	 * @return DATA extends com.cim.idm.framework.orm.info.DataInfo
	 * @throws FrameworkErrorSignal
	 * @test
	 * @code
	 * @endcode
	 */
	public DATA recover(DATA data, HDATA historyData) throws FrameworkErrorSignal
	{
		Map<String, Object> dataFields = ObjectUtil.getFieldValueMapOnlyData(data); 
		
		
		for (String fieldName : dataFields.keySet())
		{
			Object value = dataFields.get(fieldName);
			
			if ( fieldName.equals("key")) // looking same field name and assign value ?
			{ 
				Map<String, Object> keyFields = ObjectUtil.getFieldValueMapOnlyKey(historyData.getKey());
				for(String keyFieldName : keyFields.keySet())
				{
					if(keyFieldName.equalsIgnoreCase("timekey"))
						continue;
					if(ObjectUtil.getFieldValue(data.getKey(), keyFieldName) == null)
						ObjectUtil.setFieldValue(data.getKey(), keyFieldName, keyFields.get(keyFieldName));
				}
			}

			if (fieldName.startsWith("create"))
				continue;

			if (fieldName.equalsIgnoreCase("lastEventTimeKey"))
			{
				ObjectUtil.copyField(historyData.getKey(), "timeKey", data, fieldName);
			}
			else if (fieldName.startsWith("lastEvent"))
			{
				ObjectUtil.copyField4HistoryAdaptor(historyData, StringUtils.uncapitalize(fieldName.substring(4)),
					data, fieldName);
			}
			else
			{
				ObjectUtil.copyField4HistoryAdaptor(historyData, fieldName, data, fieldName);
			}
		}

		try
		{
			Map<String, String> userColumns = ObjectUtil.getUserColumsValue(historyData);
			if(userColumns != null && userColumns.size() > 0)
				ObjectUtil.setUserColumsToData(data, userColumns);
		} catch (Exception e)
		{}

		return data;
	}

	/**
	 * 
	 * @param oldData
	 *            extends com.cim.idm.framework.orm.info.DataInfo
	 * @param newData
	 *            extends com.cim.idm.framework.orm.info.DataInfo
	 * @param HistoryData
	 *            extends com.cim.idm.framework.orm.info.DataInfo
	 * @return HDATA extends com.cim.idm.framework.orm.info.DataInfo
	 * @throws FrameworkErrorSignal
	 * @test
	 * @code
	 * @endcode
	 */
	public HDATA setHV(DATA oldData, DATA newData, HDATA historyData) throws FrameworkErrorSignal
	{
		// eventUser, eventComment, eventFlag, eventTime, eventName
		String historyTableName = historyData.getClass().getSimpleName();

		//History fields
		List<String> historyFields = ObjectUtil.getFieldNamesWithKey(historyData);

		// History validation
		if (historyFields == null || historyFields.size() == 0)
			throw new FrameworkErrorSignal(ExceptionKey.NoValidHistoryDataObject_Exception, historyTableName);

		// Data fields 
		
		// if the same Field,Copy
		ObjectUtil.copyFieldNotEqual(newData.getKey(), historyData.getKey());

		for (String historyFieldName : historyFields)
		{

			Field dataField = ObjectUtil.getField(oldData, historyFieldName);
			
			//key field skip?
//			if (dataAttribute != null && "y".equalsIgnoreCase(dataAttribute.getPrimaryKeyFlag()))
//				continue;

			if (historyFieldName.toLowerCase().startsWith("old"))
				ObjectUtil.copyField(oldData, OrmStandardEngineUtil.getFieldName(historyFieldName.substring(3,
					historyFieldName.length())), historyData, ObjectUtil.getChangeField(historyFieldName));
			else if (historyFieldName.toLowerCase().startsWith("event"))
				ObjectUtil.copyField(newData, "last" + OrmStandardEngineUtil.getMethodName(historyFieldName),
						historyData, ObjectUtil.getChangeField(historyFieldName));
			else if (historyFieldName.toLowerCase().startsWith("timekey"))
				ObjectUtil.copyField(newData, "lastEventTimeKey", historyData.getKey(),
					ObjectUtil.getChangeField(historyFieldName));
			else
				ObjectUtil.copyField4HistoryAdaptor(newData, historyFieldName, historyData, historyFieldName);
		}

		return historyData;
	}

}
