package com.cim.idm.framework.orm.service;

import java.sql.Timestamp;
import java.util.List;

import com.cim.idm.framework.exception.ext.DuplicateNameSignal;
import com.cim.idm.framework.exception.ext.FrameworkErrorSignal;
import com.cim.idm.framework.exception.ext.InvalidStateTransitionSignal;
import com.cim.idm.framework.exception.ext.NotFoundSignal;
import com.cim.idm.framework.orm.info.HistoryDataInfo;
import com.cim.idm.framework.orm.info.HistoryKeyInfo;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.orm.service
 *  NAME    : CommonHistoryService.java
 *  TYPE    : JAVA
 *  DESCRIPTION : Common interface for History-related Database Handling
 *
 ****************************************************************************
 */

public interface CommonHistoryService<HKEY extends HistoryKeyInfo,HDATA extends HistoryDataInfo> extends CommonService<HKEY, HDATA>
{

	public HDATA selectLastOne(HKEY keyInfo)
			throws NotFoundSignal, InvalidStateTransitionSignal, DuplicateNameSignal, FrameworkErrorSignal;

	public HDATA selectLastOneByEventName(HKEY keyInfo, String eventName)
			throws NotFoundSignal, InvalidStateTransitionSignal, DuplicateNameSignal, FrameworkErrorSignal;

	public List<HDATA> selectBetweenTime(HKEY keyInfo, Timestamp fromTime, Timestamp toTime)
			throws NotFoundSignal, InvalidStateTransitionSignal, DuplicateNameSignal, FrameworkErrorSignal;

	public List<HDATA> selectBetweenTimeByEventName(HKEY keyInfo, Timestamp fromTime, Timestamp toTime, String eventName)
			throws NotFoundSignal, InvalidStateTransitionSignal, DuplicateNameSignal, FrameworkErrorSignal;

	public int deleteLastOne(HKEY keyInfo)
			throws NotFoundSignal, InvalidStateTransitionSignal, DuplicateNameSignal, FrameworkErrorSignal;

}
