package com.cim.idm.framework.orm.service;

import java.util.List;

import com.cim.idm.framework.exception.IDMFrameDBErrorSignal;
import com.cim.idm.framework.orm.info.DataInfo;
import com.cim.idm.framework.orm.info.KeyInfo;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.orm.service
 *  NAME    : CommonService.java
 *  TYPE    : JAVA
 *  DESCRIPTION : Common interface for the Database Handling
 *
 ****************************************************************************
 */

public interface CommonService<KEY extends KeyInfo, DATA extends DataInfo>
{
	/**
	 * <pre>select a List of DataInfo base on condition and bindSet
	 *the number of "?" and its sequence should be accordance with the bindSet array
	 *</pre>
	 * @param condition java.lang.String
	 * @param bindSet java.lang.Object[]
	 * @param clazz the DataInfo's Class
	 * @return a List of DataInfo
	 * @throws IDMFrameDBErrorSignal
	 */
	public List<DATA> select(String condition, Object[] bindSet, Class<?> clazz) throws IDMFrameDBErrorSignal;

	public DATA selectByKey(KEY keyInfo) throws IDMFrameDBErrorSignal;
	
	public DATA[] selectByKeyForBatch(KEY[] keyInfoList) throws IDMFrameDBErrorSignal;
	
	public List<DATA> selectByKey(List<KEY> keyInfoList) throws IDMFrameDBErrorSignal;
	
	public DATA selectByKeyForUpdate(KEY keyInfo) throws IDMFrameDBErrorSignal;
	
	public DATA[] selectByKeyForUpdateForBatch(KEY[] keyInfoList) throws IDMFrameDBErrorSignal;

	public int update(DATA dataInfo) throws IDMFrameDBErrorSignal;
	
	public int[] update(List<DATA> dataInfoList) throws IDMFrameDBErrorSignal;
	
	public int[] update(DATA[] dataInfoList) throws IDMFrameDBErrorSignal;
	/**
	 * <pre>update the dataInfo to DB base on condition and bindSet
	 *the number of "?" and its sequence should be accordance with the bindSet array  
	 *</pre>
	 * @param dataInfo com.cim.idm.framework.orm.info.DataInfo
	 * @param condition java.lang.String
	 * @param bindSet java.lang.Object[]
	 * @return the number of rows affected
	 * @throws IDMFrameDBErrorSignal
	 */
	public int update(DATA dataInfo, String condition, Object[] bindSet) throws IDMFrameDBErrorSignal;

	public int insert(DATA dataInfo) throws IDMFrameDBErrorSignal;
	
	public int[] insert(List<DATA> dataInfoList) throws IDMFrameDBErrorSignal;
	
	public int[] insert( DATA[] dataInfoList ) throws IDMFrameDBErrorSignal;

	public int delete(KEY keyInfo) throws IDMFrameDBErrorSignal;
	
	public int[] delete(List<KEY> keyInfoList) throws IDMFrameDBErrorSignal;
	
	public int[] delete(KEY[] keyInfoList) throws IDMFrameDBErrorSignal;

	public int delete(String condition, Object[] bindSet, Class<?> clazz) throws IDMFrameDBErrorSignal;
}
