package com.cim.idm.framework.orm.support;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cim.idm.framework.exception.ErrorSignal;
import com.cim.idm.framework.exception.IDMFrameErrorSignal;
import com.cim.idm.framework.orm.SqlTemplate;
import com.cim.idm.framework.orm.info.DataInfo;
import com.cim.idm.framework.orm.info.KeyInfo;
import com.cim.idm.framework.util.object.ObjectUtil;

public class OrmStandardEngineUtil
{
	private static Log	log	= LogFactory.getLog(OrmStandardEngineUtil.class);
	
	

	public static String getTableName(Object dataKeyInfo)
	{
		String tableName = dataKeyInfo.getClass().getSimpleName();
		
		if (dataKeyInfo instanceof KeyInfo)
			tableName = tableName.substring(0, tableName.length() - 3);
		
		return tableName.toUpperCase();
	}

	public static String getServiceName(String serviceName)
	{
		if (serviceName.endsWith("Impl"))
			return serviceName.substring(0, serviceName.length() - 4);
		
		return serviceName;
	}

	public static String getTableNameByClassName(Class<?> clazz)
	{
		String tableName = clazz.getSimpleName();
		
		if (tableName.endsWith("Key"))
			tableName = tableName.substring(0, tableName.length() - 3);
		else if (tableName.endsWith("ServiceImpl"))
			tableName = tableName.substring(0, tableName.length() - 11);
		else if (tableName.endsWith("Service"))
			tableName = tableName.substring(0, tableName.length() - 7);

		return tableName.toUpperCase();
	}

	public static String getMethodName(String attributeName)
	{
		String a = attributeName.substring(0, 1);
		String b = attributeName.substring(1, attributeName.length());
		
		return a.toUpperCase() + b;
	}

	public static String getFieldName(String attributeName)
	{
		String a = attributeName.substring(0, 1);
		String b = attributeName.substring(1, attributeName.length());
		
		return a.toLowerCase() + b;
	}

	public static String getPolicyPackageName(DataInfo dataInfo, String className)
	{
		String dataClassName = dataInfo.getClass().getName();
		String[] names = org.springframework.util.StringUtils.delimitedListToStringArray(dataClassName, ".");
		String policyPackageName = "";
		
		for (int i = 0; i < names.length; i++)
		{
			if (names[i].equalsIgnoreCase("data"))
			{
				if (className == null || className.length() == 0)
					return policyPackageName + "action";
				else
					return policyPackageName + "action" + "." + className;
			}
			
			policyPackageName = policyPackageName + names[i] + ".";
		}
		
		return policyPackageName;
	}

	public static String getHistoryPackageName(DataInfo dataInfo)
	{
		String dataClassName = dataInfo.getClass().getName();
		String[] names = org.springframework.util.StringUtils.delimitedListToStringArray(dataClassName, ".");
		String policyPackageName = "";
		
		for (int i = 0; i < names.length; i++)
		{
			if (names[i].equalsIgnoreCase("data"))
				return policyPackageName + dataInfo.getClass().getSimpleName() + "HistoryService";
			
			policyPackageName = policyPackageName + names[i] + ".";
		}
		
		return policyPackageName;
	}

	public static String getHistoryAdaptorPackageName(DataInfo dataInfo)
	{
		return getPolicyPackageName(dataInfo, null) + ".util." + dataInfo.getClass().getSimpleName() + "HistoryDataAdaptor";
	}

	public static String getVersionAttributeName(KeyInfo keyInfo)
	{
		Field[] fields = keyInfo.getClass().getDeclaredFields();
		
		for (int i = 0; i < fields.length; i++)
		{
			if (fields[i].getName().toLowerCase().endsWith("version"))
				return fields[i].getName();
		}
		
		return null;
	}

	public static String getSpecNameAttributeName(KeyInfo keyInfo)
	{
		Field[] fields = keyInfo.getClass().getDeclaredFields();
		
		for (int i = 0; i < fields.length; i++)
		{
			if (!fields[i].getName().toLowerCase().endsWith("version"))
			{
				if (!fields[i].getName().equalsIgnoreCase("factoryname"))
					return fields[i].getName();
			}
		}
		
		return null;
	}

	public static DataInfo createDataInfo(Object dataObject)
	{
		Class<?> clazz = null;
		DataInfo createdInfo = null;
		
		try
		{
			if (dataObject instanceof DataInfo)
				clazz = Class.forName(dataObject.getClass().getName());
			else if (dataObject instanceof KeyInfo)
				clazz = Class.forName(dataObject.getClass().getName().substring(0, dataObject.getClass().getName().length() - 3));
			
			createdInfo = (DataInfo) clazz.newInstance();
		} 
		catch (Exception e)
		{
			log.warn(e, e);
		}

		KeyInfo createdKeyInfo = OrmStandardEngineUtil.getKeyInfo(createdInfo);
		
		if (createdKeyInfo == null)
		{
			try
			{
				Class<?> keyClazz = null;
				if (dataObject instanceof DataInfo)
					keyClazz = Class.forName(dataObject.getClass().getName() + "Key");
				else if (dataObject instanceof KeyInfo)
					keyClazz = Class.forName(dataObject.getClass().getName());
				
				createdKeyInfo = (KeyInfo) keyClazz.newInstance();
			} 
			catch (Exception e)
			{
				log.warn(e, e);
			}
			
			ObjectUtil.setFieldValue(createdInfo, "Key", createdKeyInfo);
		}

		if (dataObject instanceof KeyInfo){
			Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(dataObject);
			for( String key : keyMap.keySet())
			{
				try{
					Field field = createdKeyInfo.getClass().getDeclaredField(key);
					Object value = keyMap.get(key);
					ObjectUtil.copyFieldValue(field,createdKeyInfo,value);
				}
				catch (Exception e)
				{
					if (log.isDebugEnabled())
						log.warn(e, e);
					else
						log.warn(e);
				}
			}
//			ObjectUtil.copyField(createdInfo, "key", dataObject);
		}
			

		return createdInfo;
	}
	
	public static DataInfo[] createDataInfoList(Object[] dataObjectList)
	{
		Class<?> clazz = null;
		
		if(dataObjectList == null || dataObjectList.length == 0)
			return null;
					
		List<DataInfo> createDataList = new ArrayList<DataInfo>();
		//find class
		try
		{
			if (dataObjectList[0] instanceof DataInfo) 
			{
				clazz = Class.forName(dataObjectList[0].getClass().getName());//clazz is about datainfo
				
			}
			else if (dataObjectList[0] instanceof KeyInfo)
				clazz = Class.forName(dataObjectList[0].getClass().getName().substring(0, dataObjectList[0].getClass().getName().length() - 3));//clazz is about datainfo
			
			for(Object obj : dataObjectList)
			{
				DataInfo createdInfo = null;
				createdInfo = (DataInfo) clazz.newInstance();
			
				KeyInfo createdKeyInfo = OrmStandardEngineUtil.getKeyInfo(createdInfo);
			
				if (createdKeyInfo == null)
				{
					try
					{
						Class<?> keyClazz = null;
						if (obj instanceof DataInfo)
							keyClazz = Class.forName(obj.getClass().getName() + "Key");//keyClazz is about keyinfo
						else if (obj instanceof KeyInfo)
							keyClazz = Class.forName(obj.getClass().getName());
						
						createdKeyInfo = (KeyInfo) keyClazz.newInstance();
					} 
					catch (Exception e)
					{
						log.warn(e, e);
					}
					ObjectUtil.setFieldValue(createdInfo, "Key", createdKeyInfo);
				}

				if (obj instanceof KeyInfo)
					ObjectUtil.copyField(createdInfo, "key", obj);
				
				createDataList.add(createdInfo);
			}
			
		} 
		catch (Exception e)
		{
			log.warn(e, e);
		}
		
		DataInfo[] dataInfoArray = null;
		if(createDataList.size()>0) {
			DataInfo[] dataInfoArrayTemp = (DataInfo[])Array.newInstance(createDataList.get(0).getClass(), createDataList.size());
			dataInfoArray = createDataList.toArray(dataInfoArrayTemp);
		}
		return dataInfoArray;
	}

	public static DataInfo createDataInfo(Class clazz) throws IDMFrameErrorSignal
	{
		String className = clazz.getName();
		
		if (className.endsWith("Key"))
			className = className.substring(0, className.length() - 3);
		else if (className.endsWith("ServiceImpl"))
			className = className.substring(0, className.length() - 11);
		else if (className.endsWith("Service"))
			className = className.substring(0, className.length() - 7);

		className = className.replaceFirst(".impl.", ".data.");
		
		try
		{
			return (DataInfo) Class.forName(className).newInstance();
		} 
		catch (Exception e)
		{
			throw new IDMFrameErrorSignal(ErrorSignal.UnexprectedSignal, e);
		}
	}

	public static KeyInfo getKeyInfo(Object dataInfo)
	{
		if (dataInfo instanceof DataInfo)
		{
			try
			{
				return ((DataInfo) dataInfo).getKey();
			} 
			catch (Exception e)
			{
				log.warn(e, e);
			}
			
			return null;
		}
		else if (dataInfo instanceof KeyInfo)
			return (KeyInfo) dataInfo;
		else
			return null;
	}




//	public static Object getValueDataType(ObjectAttributeDef objectAttributeDef, String value)
//	{
//		if (objectAttributeDef.getDataType().equalsIgnoreCase("String"))
//			return value;
//		else if (objectAttributeDef.getDataType().equalsIgnoreCase("Double"))
//		{
//			if ( value != null )
//				return Double.parseDouble(value);
//			else
//				return null;
//		}
//		else if (objectAttributeDef.getDataType().equalsIgnoreCase("Long"))
//		{
//			int index = StringUtils.indexOf(value, ".");
//			if (index >= 0)
//				value = StringUtils.substring(value, 0, index);
//			
//			if ( value != null )
//				return Long.parseLong(value);
//			else return null;
//		}
//		else if (objectAttributeDef.getDataType().equalsIgnoreCase("TimeStamp"))
//			return TimeStampUtil.getTimestamp(value);
//		
//		return "";
//	}

	public static Class<?>[] getClassType(Object[] args)
	{
		Class<?>[] classTypes = null;
		if (args != null)
		{
			classTypes = new Class[args.length];
			for (int i = 0; i < args.length; i++)
			{
				if (args[i] != null)
				{
					if (args[i].getClass().getSimpleName().equalsIgnoreCase("integer"))
						classTypes[i] = Integer.TYPE;
					else if (args[i].getClass().getSimpleName().equalsIgnoreCase("boolean"))
						classTypes[i] = Boolean.TYPE;
					else if (args[i].getClass().getSimpleName().equalsIgnoreCase("byte"))
						classTypes[i] = Byte.TYPE;
					else if (args[i].getClass().getSimpleName().equalsIgnoreCase("double"))
						classTypes[i] = Double.TYPE;
					else if (args[i].getClass().getSimpleName().equalsIgnoreCase("long"))
						classTypes[i] = Long.TYPE;
					else
					{
						if (args[i] instanceof BigDecimal)
						{
							classTypes[i] = Long.TYPE;
							args[i] = Long.valueOf(args[i].toString());
						}
						else
							classTypes[i] = args[i].getClass();
					}
				}
			}
		}
		
		return classTypes;
	}


	public static Object ormExecute(Object dataObject, List<Map<String,Object>> resultList) throws IDMFrameErrorSignal
	{
		List<DataInfo> resultDataInfos = new ArrayList<DataInfo>();
		
		String keyName = "";
		for (int j = 0; j < resultList.size(); j++)
		{
			try {
				boolean bTimeStamp = false;
				DataInfo data = createDataInfo(dataObject);
				
				Map<String, Object> map = ObjectUtil.getFieldValueMapOnlyData(data);
				Map<String, Object> orderMap =  resultList.get(j);			
				for( String key : map.keySet() )
				{   
					keyName = key;
					if (StringUtils.equalsIgnoreCase(key, "key")) continue;
					Field field = ObjectUtil.getField(data, key);
					if (field.getType().getClass().equals(java.sql.Timestamp.class))
						bTimeStamp = true;
					
					
					Object resultValue = orderMap.get(key.toLowerCase());				
					if (resultValue != null) {
						try {
							ObjectUtil.setFieldValue(data, key, resultValue);
						} catch (NullPointerException ex) {
							ex.printStackTrace();
						}
					}
					else if (resultValue == null && bTimeStamp)
						ObjectUtil.setFieldValue(data, key, null);				
				}
				Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(data);
				for( String key : keyMap.keySet())
				{
					keyName = key;
					Object resultValue = orderMap.get(key);				
					OrmStandardEngineUtil.setPropertyValueIntoKeyInfo(data, key, resultValue);
				}
				resultDataInfos.add(data);
			} catch (Exception e) {
				e.printStackTrace();
				log.error(keyName);
			}
		}
		
		if (resultList.size() > 1)
			return resultDataInfos;
		else
			return resultDataInfos.get(0);
		
	}
	
	public static DataInfo[] ormExecuteForBatch(Object[] dataObject, List<Map<String,Object>> resultList) throws IDMFrameErrorSignal
	{
		List<DataInfo> resultDataInfos = new ArrayList<DataInfo>();
		
		String keyName = "";
		for (int j = 0; j < resultList.size(); j++)
		{
			try {
				boolean bTimeStamp = false;
				DataInfo data = createDataInfo(dataObject[j]);
				
				Map<String, Object> map = ObjectUtil.getFieldValueMapOnlyData(data);
				Map<String,Object> orderMap =  resultList.get(j);			
				for( String key : map.keySet() )
				{   
					keyName = key;
					if (StringUtils.equalsIgnoreCase(key, "key")) continue;
					Field field = ObjectUtil.getField(data, key);
					if (field.getType().getClass().equals(java.sql.Timestamp.class))
						bTimeStamp = true;
					
					
					Object resultValue = orderMap.get(key.toLowerCase());				
					if (resultValue != null) {
						try {
							ObjectUtil.setFieldValue(data, key, resultValue);
						} catch (NullPointerException ex) {
							ex.printStackTrace();
						}
					}
					else if (resultValue == null && bTimeStamp)
						ObjectUtil.setFieldValue(data, key, null);				
				}
				Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(data);
				for( String key : keyMap.keySet())
				{
					keyName = key;
					Object resultValue = orderMap.get(key);				
					OrmStandardEngineUtil.setPropertyValueIntoKeyInfo(data, key, resultValue);
				}
				resultDataInfos.add(data);
			} catch (Exception e) {
				e.printStackTrace();
				log.error(keyName);
			}
		}
		
		DataInfo[] resultDataArray = (DataInfo[]) Array.newInstance(resultDataInfos.get(0).getClass(), resultDataInfos.size());
		resultDataArray = resultDataInfos.toArray(resultDataArray);
		return resultDataArray;
		
	}


	private static void setPropertyValueIntoKeyInfo(DataInfo dataInfo, String fieldName, Object value)
	{
		try
		{
			ObjectUtil.setFieldValue(getKeyInfo(dataInfo), fieldName, value);
		} 
		catch (Exception e)
		{
			log.warn(e, e);
		}
	}

	public static String generateSqlStatement(String queryType, Object dataKeyObject, List<Object> objectValue) throws IDMFrameErrorSignal
	{
		StringBuilder strBuilder = new StringBuilder();

		String tableName = OrmStandardEngineUtil.getTableName(dataKeyObject);
		
		if (queryType.equalsIgnoreCase("select"))
			strBuilder.append( "select * from " ).append( tableName );
		else if (queryType.equalsIgnoreCase("select.for.update"))
		{
			strBuilder.append( "select * from " ).append( tableName );
		}
		else if (queryType.equalsIgnoreCase("delete"))
			strBuilder.append( "delete from " ).append( tableName );
		else if (queryType.equalsIgnoreCase( "update" ))
			strBuilder.append( "update " ).append( tableName ).append( " set " );
		else if (queryType.equalsIgnoreCase("insert"))
			strBuilder.append( "insert " ).append( "into " ).append(tableName);

		generateCondition( strBuilder, queryType, tableName, dataKeyObject, objectValue);
		if ( queryType.equalsIgnoreCase("select.for.update"))
			strBuilder.append( " for update" );
		
		return strBuilder.toString();
	}
	
	/**
	 * <p>generate a query Sql String , execute this sql can get a ResultSet contain a list of DataInfo with a list keyInfo parameter dataKeyObjectList</p>
	 * <b>note:</b><em>&nbsp;&nbsp;this sql can not ensure the sequence of DataInfo in ResultSet </em>
	 * @param queryType
	 * @param dataKeyObjectList
	 * @param objectValue
	 * @return
	 * @throws IDMFrameErrorSignal
	 */
	@Deprecated
	public static String generateSqlStatementForBatch(String queryType, Object[] dataKeyObjectList, List<Object> objectValue) throws IDMFrameErrorSignal
	{
		StringBuilder strBuilder = new StringBuilder();

		String tableName = OrmStandardEngineUtil.getTableName(dataKeyObjectList[0]);
		
		strBuilder.append( "select * from " ).append( tableName );
		
		generateConditionForBatch( strBuilder, tableName, dataKeyObjectList, objectValue);
		
		if(queryType.equalsIgnoreCase("select.for.update"))
			strBuilder.append( " for update" );
		
		return strBuilder.toString();
	}

	public static Object[] getPrimaryNoTimeKeyForHistory(KeyInfo keyInfo, int duplicateCount)
	{
		List objectValue = new ArrayList();
		Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(keyInfo);
		for (int j = 0; j < duplicateCount; j++)
		{
			for (String key : keyMap.keySet())
			{
				if (!key.equalsIgnoreCase("timekey"))
					objectValue.add(keyMap.get(key));
			}
		}
		return objectValue.toArray();
	}
	
	
	
	public static String generateExtendedSqlStatement(String queryType, Object dataObject, String queryDetailType, String sql) throws IDMFrameErrorSignal
	{
		StringBuilder strBuilder = new StringBuilder( sql );
		String tableName = getTableName(dataObject);
		
		if (queryType.equalsIgnoreCase("select") || queryType.equalsIgnoreCase("delete"))
		{
			if (queryDetailType.equalsIgnoreCase("selectLastOne") || queryDetailType.equalsIgnoreCase("selectLastOneByEventName") || queryDetailType.equalsIgnoreCase("deleteLastOne"))
			{
				strBuilder.delete(strBuilder.length() - 1, strBuilder.length());
				strBuilder.append( "(select " );
				
				strBuilder.append( "/*+ index_desc(" ).append( tableName ).append( ", " ).append( tableName.toUpperCase()).append( "_PK) */ " );
				
				strBuilder.append( "timekey " );
				strBuilder.append( "from " ).append( tableName );
				
				strBuilder.append( " where " );
				
				Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(dataObject);
				for (String key : keyMap.keySet())
				{
					if (!key.equalsIgnoreCase("timekey"))
						strBuilder.append(key).append("=").append(" ? ").append(" and ");
				}				
				
				if (strBuilder.toString().endsWith(" and "))
					strBuilder.delete(strBuilder.length() - 5, strBuilder.length());
				else if (strBuilder.toString().endsWith(" where "))
					strBuilder.delete(strBuilder.length() - 7, strBuilder.length());

				if (queryDetailType.equalsIgnoreCase("selectLastOneByEventName"))
					strBuilder.append("and eventName = ? ");
					
				strBuilder.append( "and rownum = 1)" );
			}
			else if (queryDetailType.equalsIgnoreCase("selectBetweenTime"))
			{
				strBuilder.delete(strBuilder.length() - 10, strBuilder.length());
				strBuilder.append(" (eventTime between ? and ?) ");
				
//				if (cancelFalg == null)
				strBuilder.append("order by timekey");
//				else
//					strBuilder.append("and cancelFlag='" + cancelFalg + "'" + " order by timekey");
			}
			else if (queryDetailType.equalsIgnoreCase("selectBetweenTimeByEventName"))
			{
				strBuilder.delete(strBuilder.length() - 10, strBuilder.length());
				strBuilder.append(" (eventTime between ? and ?) ");
				
//				if (cancelFalg == null)
				strBuilder.append("and eventName = ? " + " order by timekey");
//				else
//					strBuilder.append("and cancelFlag='" + cancelFalg + "'" + " and eventName = ? " + " order by timekey");
			}
		}
		else if (queryType.equalsIgnoreCase("update"))
		{
			/*if (queryDetailType.equalsIgnoreCase("updateWithCancelFlag"))
			{
				strBuilder.append("update " + tableName + " set cancelFlag = ? ");
				strBuilder.append(" where ");
				
				Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(dataObject);
				for (String key : keyMap.keySet())
				{
					strBuilder.append(key).append("=").append(" ? ").append(" and ");
				}				
				
				if (strBuilder.toString().endsWith(" and "))
					strBuilder.delete(strBuilder.length() - 5, strBuilder.length());
				else if (strBuilder.toString().endsWith(" where "))
					strBuilder.delete(strBuilder.length() - 7, strBuilder.length());
			}*/
		}

		return strBuilder.toString();
	}

	public static String generateSqlStatement(String queryType, Object dataKeyObject) throws IDMFrameErrorSignal
	{
		StringBuilder strBuilder = new StringBuilder();

		String tableName = OrmStandardEngineUtil.getTableName(dataKeyObject);
		
		if (queryType.equalsIgnoreCase("select"))
			strBuilder.append( "select * from " ).append( tableName );
		else if (queryType.equalsIgnoreCase("select.for.update"))
		{
			strBuilder.append( "select * from " ).append( tableName );
		}
		else if (queryType.equalsIgnoreCase("delete"))
			strBuilder.append( "delete from " ).append( tableName );
		else if (queryType.equalsIgnoreCase( "update" ))
			strBuilder.append( "update " ).append( tableName ).append( " set " );
		else if (queryType.equalsIgnoreCase("insert"))
			strBuilder.append( "insert " ).append( "into " ).append(tableName);

		generateCondition( strBuilder, queryType, tableName, dataKeyObject , null );
		
		if ( queryType.equalsIgnoreCase("select.for.update"))
			strBuilder.append( " for update" );
		
		return strBuilder.toString();
	}
	
	public static String getConditionSql(String sql, String condition)
	{
		if (condition.toLowerCase().trim().startsWith("where "))
			sql = sql + " " + condition.trim();
		else
			sql = sql + " where " + condition;
		
		return sql;
		
	}

	public static List<Object> getBindSetObject(String queryType, Object dataKeyObject) throws IDMFrameErrorSignal
	{
		List<Object> objectValue = new ArrayList<Object>();
		if (queryType.equalsIgnoreCase("select") || queryType.equalsIgnoreCase("select.for.update") || queryType.equalsIgnoreCase("delete"))
		{
			Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(dataKeyObject);			
			for( String key : keyMap.keySet() )
			{
				objectValue.add(keyMap.get(key));
			}
		}
		else if (queryType.equalsIgnoreCase("update"))
		{
			Map<String, Object> map = ObjectUtil.getFieldValueMap((DataInfo)dataKeyObject);			
			
			for( String key : map.keySet() )
			{            	
				if (StringUtils.equalsIgnoreCase(key, "key")) continue;
				objectValue.add(map.get(key));
			}
			
			Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey((DataInfo)dataKeyObject);			
			for( String key : keyMap.keySet() )
			{
				objectValue.add(keyMap.get(key));
			}
		}
		else if (queryType.equalsIgnoreCase("insert")) 
		{
			Map<String, Object> map = ObjectUtil.getFieldValueMap((DataInfo)dataKeyObject);			
			
			for( String key : map.keySet() )
			{            	
				if (StringUtils.equalsIgnoreCase(key, "key")) continue;
				Field field = ObjectUtil.getField(dataKeyObject, key);
				objectValue.add(map.get(key));									
			}
		}
		else 
			throw new IDMFrameErrorSignal(ErrorSignal.InvalidQueryType, queryType);		
		
		return objectValue;
	}
	
	public static void generateCondition(StringBuilder strBuilder, String queryType, String tableName, Object dataKeyObject, List<Object> objectValue) throws IDMFrameErrorSignal
	{
		
		if (queryType.equalsIgnoreCase("select") || queryType.equalsIgnoreCase("select.for.update") || queryType.equalsIgnoreCase("delete"))
		{
			strBuilder.append(" where ");
			Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(dataKeyObject);
			int nCountNullValues = 0;
			boolean hasTimeKey = false;
			//tbrbkang 2013.02.20, to change where condition's order. key value of null will be located in rear side.
			for( String key : keyMap.keySet() )
			{
				if(key.equalsIgnoreCase("timekey")){
					hasTimeKey = true;
					continue;
				}
				if(keyMap.get(key) != null)
				{
					strBuilder.append(key.toUpperCase()).append("=").append("?").append(" and ");		
					if (objectValue != null)
						objectValue.add(keyMap.get(key));
				}
				else
				{
					strBuilder.append(key.toUpperCase()).append(" is null ").append(" and ");
					nCountNullValues++;
				}
			}
			if(hasTimeKey){
				strBuilder.append("TIMEKEY=?");
				if(keyMap.get("timeKey")!=null ){
					if (objectValue != null)
						objectValue.add(keyMap.get("timeKey"));
				}else if(keyMap.get("timekey")!=null){
					if (objectValue != null)
						objectValue.add(keyMap.get("timekey"));
				}else if(keyMap.get("TimeKey")!=null){
					if (objectValue != null)
						objectValue.add(keyMap.get("TimeKey"));
				}else{
					for( String key : keyMap.keySet() ){
						if(key.equalsIgnoreCase("timekey"))
							if(keyMap.get(key) != null)
							{
								strBuilder.append(key.toUpperCase()).append("=").append("?");		
								if (objectValue != null)
									objectValue.add(keyMap.get(key));
							}
					}
				}
			}
			
			if (strBuilder.toString().endsWith(" and "))
				strBuilder.delete(strBuilder.length() - 5, strBuilder.length());
			else if (strBuilder.toString().endsWith(" where "))
				strBuilder.delete(strBuilder.length() - 7, strBuilder.length());
			
			if(nCountNullValues > 1)
			{
				log.warn("null key values can't be more than 1, it can cause unexepected result, SQL=" +  strBuilder.toString());
			}
		}
		else if (queryType.equalsIgnoreCase("update"))
		{
			String where = " where ";
			Map<String, Object> map = ObjectUtil.getFieldValueMap((DataInfo)dataKeyObject);			
			
			for( String key : map.keySet() )
			{            	
				if (StringUtils.equalsIgnoreCase(key, "key")) continue;
				strBuilder.append(key.toUpperCase()).append("=").append("?").append(",");	
				if (objectValue != null)
					objectValue.add(map.get(key));
			}
			
			Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey((DataInfo)dataKeyObject);			
			for( String key : keyMap.keySet() )
			{
				where = where + key.toUpperCase() + "=" + "?" + " and ";				
				if (objectValue != null)
					objectValue.add(keyMap.get(key));
			}
			where = where.substring(0, where.length() - 5);			
			strBuilder.delete(strBuilder.length() - 1, strBuilder.length()).append(where);			
			

		}
		else if (queryType.equalsIgnoreCase("insert"))
		{
			StringBuilder strBuilder2 = new StringBuilder();
			strBuilder.append(" (");
			Map<String, Object> map = ObjectUtil.getFieldValueMap((DataInfo)dataKeyObject);			
			
			for( String key : map.keySet() )
			{            	
				if (StringUtils.equalsIgnoreCase(key, "key")) continue;
				strBuilder.append(key.toUpperCase()).append(",");	
				Field field = ObjectUtil.getField(dataKeyObject, key);

				if (field != null && field.getType().getClass().equals(java.sql.Timestamp.class) && StringUtils.equals(key.toLowerCase(), "systemtime"))
				{
					if("ORACLE".equalsIgnoreCase(SqlTemplate.RDB_TYPE))
						strBuilder2.append("SYSDATE").append(",");
					else if("MySQL".equalsIgnoreCase(SqlTemplate.RDB_TYPE))
						strBuilder2.append("CURRENT_TIMESTAMP() ").append(","); 
					else if("Microsoft SQL Server".equalsIgnoreCase(SqlTemplate.RDB_TYPE))
						strBuilder2.append("getDate() ").append(",");
					else if("PostgreSQL".equals(SqlTemplate.RDB_TYPE))
						strBuilder2.append("current_timestamp ").append(",");
										
				}
				else {
					strBuilder2.append("?").append(",");								
					if (objectValue != null)
						objectValue.add(map.get(key));									
				}
			}			
			strBuilder.delete(strBuilder.length()-1, strBuilder.length());
			strBuilder.append(") values (");
			
			strBuilder2.delete(strBuilder2.length()-1, strBuilder2.length());
			strBuilder2.append(")");
			strBuilder.append(strBuilder2);
			
		}
		else
			throw new IDMFrameErrorSignal(ErrorSignal.InvalidQueryType, queryType);
	}
	
	public static void generateConditionForBatch(StringBuilder strBuilder, String tableName, Object[] dataKeyObject, List<Object> objectValue) throws IDMFrameErrorSignal
	{
		strBuilder.append(" where ");
		int nCountNullValues = 0;
		//tbrbkang 2013.02.20, to change where condition's order. key value of null will be located in rear side.
	
		int nOrder = 0;
		for(Object obj : dataKeyObject)
		{
			Map<String, Object> keyMap = ObjectUtil.getFieldValueMapOnlyKey(dataKeyObject[nOrder]);
			boolean hasTimeKey = false;
			int keyColumnNo = keyMap.size();
			if(keyColumnNo>0) strBuilder.append("(");
			for( String key : keyMap.keySet() )
			{
				
				if(key.equalsIgnoreCase("timekey")){
					hasTimeKey = true;
					continue;
				}

				if(keyMap.get(key) != null)
				{
					strBuilder.append(key).append("=").append("?").append(" and ");		
					if (objectValue != null)
						objectValue.add(keyMap.get(key));
				}
				else
				{
					strBuilder.append(key).append(" is null ").append(" and ");
					nCountNullValues++;
				}
			}
			if(hasTimeKey){
				strBuilder.append("timekey=?");
				if(keyMap.get("timeKey")!=null ){
					if (objectValue != null)
						objectValue.add(keyMap.get("timeKey"));
				}else if(keyMap.get("timekey")!=null){
					if (objectValue != null)
						objectValue.add(keyMap.get("timekey"));
				}else if(keyMap.get("TimeKey")!=null){
					if (objectValue != null)
						objectValue.add(keyMap.get("TimeKey"));
				}else{
					for( String key : keyMap.keySet() ){
						if(key.equalsIgnoreCase("timekey"))
							if(keyMap.get(key) != null)
							{
								strBuilder.append(key).append("=").append("?");		
								if (objectValue != null)
									objectValue.add(keyMap.get(key));
							}
					}
				}
			}
			
			if (strBuilder.toString().endsWith(" and "))
				strBuilder.delete(strBuilder.length() - 5, strBuilder.length());
			else if (strBuilder.toString().endsWith(" where "))
				strBuilder.delete(strBuilder.length() - 7, strBuilder.length());
			if(keyColumnNo>0) strBuilder.append(") or ");
			nOrder++;
		}
		
		if (strBuilder.toString().endsWith(" or "))
			strBuilder.delete(strBuilder.length() - 4, strBuilder.length());
		else if (strBuilder.toString().endsWith(" where "))
			strBuilder.delete(strBuilder.length() - 7, strBuilder.length());
			
		if(nCountNullValues > 1)
		{
			log.warn("null key values can't be more than 1, it can cause unexepected result, SQL=" +  strBuilder.toString());
		}
	}
	
	private static boolean checkNotNullValue(Object dataKeyObject, String fieldName, Object value)
	{
		Field field = ObjectUtil.getField(dataKeyObject, fieldName);
				
		if (field.getType().getClass().equals(Timestamp.class) && fieldName.equalsIgnoreCase("systemtime"))
			return true;
		
		if (value == null)
			return false;

		if (field.getType().getClass().equals(Timestamp.class))
		{
			if (value.equals(new Timestamp(0)) || value.toString().length() == 0)
				return false;
		}
		else if (value != null && (field.getType().getClass().equals(Long.class)) || field.getType().getClass().equals(Double.class))
		{
			if (value.toString().length() == 0)
				return false;
		}
		
		return true;
	}
	
	public static OrmSqlStatement getSqlPrimaryWithoutColumns(Object dataKeyInfo, Map<String, String> withoutColumns)
	{
		//String tableName = getTableName(dataKeyInfo);
		String sql = "";
		KeyInfo keyInfo = null;
		
		if (dataKeyInfo instanceof DataInfo)
			keyInfo = getKeyInfo((DataInfo) dataKeyInfo);
		else if (dataKeyInfo instanceof KeyInfo)
			keyInfo = (KeyInfo) dataKeyInfo;

		List objectValue = new ArrayList();
		
		Map<String, Object> map = ObjectUtil.getFieldValueMapOnlyKey(keyInfo);
		
		for ( String keyName : map.keySet())
		{
			if (withoutColumns == null || !withoutColumns.containsKey(keyName))
			{
				sql = sql + keyName + "=? and ";
				objectValue.add( map.get(keyName));
			}
		}

		return new OrmSqlStatement(sql.substring(0, sql.length() - 5), objectValue.toArray());
	}

	public static Object[] getPrimaryWithoutColumns(Object dataKeyInfo, Map<String, String> withoutColumns)
	{
		String sql = "";
		KeyInfo keyInfo = null;
		
		if (dataKeyInfo instanceof DataInfo)
			keyInfo = getKeyInfo((DataInfo) dataKeyInfo);
		else if (dataKeyInfo instanceof KeyInfo)
			keyInfo = (KeyInfo) dataKeyInfo;

		List objectValue = new ArrayList();
		
		Map<String, Object> map = ObjectUtil.getFieldValueMapOnlyKey(keyInfo);
		
		for ( String keyName : map.keySet())
		{
			if (withoutColumns == null || !withoutColumns.containsKey(keyName))
			{
				sql = sql + keyName + "=? and ";
				objectValue.add( map.get(keyName));
			}
		}
		
		return objectValue.toArray();
	}
	
	public static Object[] getPrimaryWithoutTimeKey(Object dataKeyInfo, int duplicate)
	{
		KeyInfo keyInfo = null;
		if (dataKeyInfo instanceof DataInfo)
			keyInfo = getKeyInfo((DataInfo) dataKeyInfo);
		else if (dataKeyInfo instanceof KeyInfo)
			keyInfo = (KeyInfo) dataKeyInfo;

		List objectValue = new ArrayList();
		
		Map<String, Object> map = ObjectUtil.getFieldValueMapOnlyKey(keyInfo);
		
		for ( String keyName : map.keySet())
		{
			if (!keyName.equalsIgnoreCase("timekey"))
			{
				objectValue.add( map.get(keyName) );
			}
		}
		
		return objectValue.toArray();
	}
	
	public static Object[] getSelectOrDeleteBindObjects(Object dataKeyInfo)
	{
		String tableName = getTableName(dataKeyInfo);

		KeyInfo keyInfo = null;
		
		if (dataKeyInfo instanceof DataInfo)
			keyInfo = getKeyInfo((DataInfo) dataKeyInfo);
		else if (dataKeyInfo instanceof KeyInfo)
			keyInfo = (KeyInfo) dataKeyInfo;

		List<Object> objectValue = new ArrayList<Object>();
		Map<String, Object> map = ObjectUtil.getFieldValueMapOnlyKey(keyInfo);
		
		for ( String keyName : map.keySet())
		{
			if (!keyName.equalsIgnoreCase("timekey"))
			{
				objectValue.add( map.get(keyName) );
			}
		}		
		return objectValue.toArray();
	}
	
	public static DataInfo[] sortDataInfoListByKeyInfoList(DataInfo[] dataInfoList, KeyInfo[] keyInfoList ) throws RuntimeException
	{
		log.debug(String.format("sort %s[%d] by %s[%d] ",dataInfoList[0].getClass().getSimpleName(),dataInfoList.length,keyInfoList[0].getClass().getSimpleName(),keyInfoList.length));
		if(dataInfoList.length!=keyInfoList.length){
			log.error(String.format("can not sort %s[%d] by %s[%d]", dataInfoList[0].getClass().getSimpleName(),dataInfoList.length,keyInfoList[0].getClass().getSimpleName(),keyInfoList.length));
			throw new RuntimeException(String.format("can not sort %s[%d] by %s[%d]", dataInfoList[0].getClass().getSimpleName(),dataInfoList.length,keyInfoList[0].getClass().getSimpleName(),keyInfoList.length));
		}
			
		DataInfo[] sortedDataInfoList = new DataInfo[dataInfoList.length];
		int i = 0;
		Map<DataInfo,Boolean> DataInfoSortMark = new HashMap<DataInfo,Boolean>();
		for(KeyInfo keyInfo : keyInfoList){
			Map<String, Object> keyValueMapByKeyInfo = ObjectUtil.getFieldValueMapOnlyKey(keyInfo);
			for(DataInfo dataInfo : dataInfoList){
				if(DataInfoSortMark.get(dataInfo)!=null && DataInfoSortMark.get(dataInfo)==true) 
					continue;
				Map<String, Object> keyValueMapByDataInfo = ObjectUtil.getFieldValueMapOnlyKey(dataInfo);
				boolean find = true;
				for( String keyInfoProp : keyValueMapByKeyInfo.keySet()){
					if(keyValueMapByDataInfo.get(keyInfoProp).getClass().equals(String.class))
						if(keyValueMapByDataInfo.get(keyInfoProp).equals(keyValueMapByKeyInfo.get(keyInfoProp)))
							continue;
						else
							find = false;
					else if(keyValueMapByDataInfo.get(keyInfoProp).getClass().isPrimitive())
						if(keyValueMapByDataInfo.get(keyInfoProp)==(keyValueMapByKeyInfo.get(keyInfoProp)))
							continue;
						else
							find = false;
					else if(keyValueMapByDataInfo.get(keyInfoProp).getClass().equals("java.util.Date") ||
							keyValueMapByDataInfo.get(keyInfoProp).getClass().equals("java.sql.Timestamp")) 
						if(((java.util.Date)keyValueMapByDataInfo.get(keyInfoProp)).getTime()==((java.util.Date)keyValueMapByKeyInfo.get(keyInfoProp)).getTime())
							continue;
						else
							find = false;
					else
						throw new RuntimeException("KeyInfo ["+ ObjectUtil.getString(keyInfo)+"] property must be string,primitive or date type ");
				}
				if(find){
					sortedDataInfoList[i] = dataInfo;
					DataInfoSortMark.put(dataInfo, true);
					break;
				}
			}
			
			i++;
		}
		DataInfoSortMark.clear();
		return sortedDataInfoList;
	}
}
