package com.cim.idm.framework.util.file;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
//import org.osgi.framework.Bundle;

import com.cim.idm.framework.util.resource.ResourceUtil;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.util.file<br>
 *  NAME    : FileUtil.java<br>
 *  TYPE    : JAVA<br>
 *  DESCRIPTION : File-related utility<br>
 *
 ****************************************************************************
 */

public class FileUtil
{

	private static Log	log	= LogFactory.getLog(FileUtil.class.getName());

	/**
	 * 
	 * 
	 * @param file
	 * @param data
	 * @param encoding
	 * @throws IOException
	 */
	public static void writeStringToFile(File file, String data, String encoding) throws IOException
	{
		if (encoding == null)
			encoding = "UTF-8";
		BufferedWriter output = new BufferedWriter(new FileWriter(file, false));
		try
		{
			output.append(data);
			output.flush();
		} finally
		{
			output.close();
		}
	}

	public static List<File> getFileList(File fileDir, String filtername, boolean watchInnerDirs) throws Exception
	{
		filtername = filtername.toUpperCase();
		if (fileDir.isFile())
		{
			List<File> list = new ArrayList<File>();
			addFile(fileDir, filtername, list);
			return list;
		}
		else
		{
			return getFilesInDirectory(fileDir, filtername, watchInnerDirs);
		}

	}

	public static List<File> getFileList(String basePath, String dir, String filtername, boolean watchInnerDirs)
			throws Exception
	{
		File file = ResourceUtil.getBundleResource(basePath, dir);
		filtername = filtername.toUpperCase();
		if (file.isFile())
		{
			List<File> list = new ArrayList<File>();
			addFile(file, filtername, list);
			return list;
		}
		else
		{
			return getFilesInDirectory(file, filtername, watchInnerDirs);
		}
	}

	public static List<String> getFileNameList(String basePath, String dir, String filtername, boolean watchInnerDirs)
			throws Exception
	{
		File file = ResourceUtil.getBundleResource(basePath, dir);
		filtername = filtername.toUpperCase();
		if (file == null)
			return new ArrayList<String>();

		if (file.isFile())
		{
			List<String> list = new ArrayList<String>();
			addFileName(file, filtername, list);
			return list;
		}
		else
		{
			return getFileNamesInDirectory(file, filtername, watchInnerDirs);
		}
	}

	private static List<File> getFilesInDirectory(File aStartingDir, String filtername, boolean watchInnerDirs)
	{
		List<File> list = new ArrayList<File>();

		filtername = filtername.toUpperCase();

		File[] filesAndDirs = aStartingDir.listFiles();
		List<File> filesDirs = Arrays.asList(filesAndDirs);

		for (Iterator<File> iter = filesDirs.iterator(); iter.hasNext();)
		{
			File file = (File) iter.next();

			if (file.isFile())
			{
				addFile(file, filtername, list);
			}
			else if (watchInnerDirs)
			{
				List<File> deeperList = getFilesInDirectory(file, filtername, watchInnerDirs);
				list.addAll(deeperList);
			}
		}
		return list;
	}

	/**
	 * 
	 * 
	 * @param aStartingDir
	 * @param filtername
	 * @param watchInnerDirs
	 * @return
	 */
	private static List<String> getFileNamesInDirectory(File aStartingDir, String filtername, boolean watchInnerDirs)
	{
		List<String> list = new ArrayList<String>();

		filtername = filtername.toUpperCase();

		File[] filesAndDirs = aStartingDir.listFiles();
		List<File> filesDirs = Arrays.asList(filesAndDirs);

		for (Iterator<File> iter = filesDirs.iterator(); iter.hasNext();)
		{
			File file = iter.next();

			if (file.isFile())
			{
				addFileName(file, filtername, list);
			}
			else if (watchInnerDirs)
			{
				List<String> deeperList = getFileNamesInDirectory(file, filtername, watchInnerDirs);
				list.addAll(deeperList);
			}
		}
		return list;
	}

	private static void addFile(File file, String afiltername, List<File> list)
	{
		if (isFileName(file, afiltername))
			list.add(file);
	}

	/**
	 * If it is a format file is specified, to be added to the list
	 * 
	 * @param file
	 * @param afiltername
	 * @param list
	 */
	private static void addFileName(File file, String afiltername, List<String> list)
	{
		if (file.getName().toUpperCase().endsWith(afiltername))
		{
			list.add(file.getName());
		}
	}

	private static boolean isFileName(File file, String afiltername)
	{
		return file.getName().toUpperCase().endsWith(afiltername);
	}

	/**
	 * write File 
	 * 
	 * @param path
	 * @param fileName
	 * @param content
	 * @return
	 */
	public static boolean writeFile(String path, String fileName, String content)
	{
		BufferedWriter writer = null;
		try
		{
			File file = new File(path);
			if (!file.exists())
				file.mkdirs();

			writer = new BufferedWriter(new FileWriter(path + File.separator + fileName));
			writer.write(content);
			file = null;

			String msg = "Completed : File(%s%s)";
			msg = String.format(msg, path, fileName);
			log.info(msg);
			return true;
		} catch (IOException e)
		{
			String msg = "Failed : File(%s%s)";
			msg = String.format(msg, path, fileName);
			log.error(msg);
			return false;
		} catch (Exception e)
		{
			log.error(e);
			return false;
		}finally{
			if(writer!=null){
				try {
					writer.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

}
