package com.cim.idm.framework.util.file;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class FileWatcher extends Thread
{
	private static Log		log					= LogFactory.getLog(FileWatcher.class);

	private String			_watchRootDir		= null;
	private String			_filterName			= null;
	private boolean			_watchInnerDirs		= false;

	private volatile Thread	_Watcher			= null;

	private List			_FileListenerList	= null;
	private Map				_FileMap			= null;
	private int				_Interval			= 2 * 1000;

	// When a file is changed Available flag if you want only one of Action while one checking end.
	public boolean			checkActionFlag		= true;

	public FileWatcher(String fileName) throws Exception
	{
		super.setName("FileWatcher");

		if (fileName == null)
		{
			log.error("FileName cannot be <null>");
			throw new NullPointerException("FileName cannot be <null>");
		}

		try
		{
			_FileMap.put(fileName, new File(fileName).lastModified());
			_FileListenerList = new Vector();
			start();
		} catch (Exception ex)
		{
			//log.error("Fail to initialize", ex); 
			throw ex;
		}
	}

	public FileWatcher(String aDirName, String aExtension, boolean watchInnerDirs) throws Exception
	{
		super.setName("FileWatcher");

		if (aDirName == null)
		{
			log.error("Directory cannot be <null>");
			throw new NullPointerException("dirName cannot be <null>");
		}

		_watchRootDir = aDirName;
		_filterName = aExtension;
		_watchInnerDirs = watchInnerDirs;

		try
		{
			_FileMap = getWatchFileInfo();

			_FileListenerList = new Vector();

			start();
		} catch (Exception ex)
		{
			//log.error("Fail to initialize", ex); 
			throw ex;
		}
	}

	public void addFileListener(FileListener aFileListener)
	{
		_FileListenerList.add(aFileListener);
	}

	public void removeFileListener(FileListener aFileListener)
	{
		this._FileListenerList.remove(aFileListener);
	}

	//	public FileListener[] getFileListeners() { return (FileListener[])_FileListenerList.toArray(); }

	public void setInterval(int aSeconds)
	{
		_Interval = aSeconds * 1000;
	}

	public void stopWatching()
	{
		_Watcher = null;
	}

	public void start()
	{
		_Watcher = new Thread(this);
		_Watcher.setDaemon(true);
		_Watcher.start();
	}

	public void run()
	{
		Thread thisThread = Thread.currentThread();
		while (thisThread == _Watcher)
		{
			try
			{
				Thread.sleep(_Interval);
			} catch (InterruptedException ex)
			{
				_Watcher = null;
			}

			checkFiles();
		}
	}

	private Map getWatchFileInfo() throws Exception
	{
		List list = FileUtil.getFileList(new File(_watchRootDir), _filterName, _watchInnerDirs);

		Map filemap = new HashMap();
		for (Iterator iter = list.iterator(); iter.hasNext();)
		{
			File file = (File) iter.next();
			filemap.put(file.getAbsolutePath(), new Long(file.lastModified()));
		}
		return filemap;
	}

	private synchronized void checkFiles()
	{
		try
		{
			checkActionFlag = false;
			Map curFiles = getWatchFileInfo();

			Map removedFiles = new HashMap();

			for (Iterator iter = _FileMap.entrySet().iterator(); iter.hasNext();)
			{
				Entry entry = (Entry) iter.next();
				String key = (String) entry.getKey();
				Long origin_modified = (Long) entry.getValue();

				Long cur_modified = (Long) curFiles.remove(key);
				if (cur_modified == null)
				{
					removedFiles.put(key, origin_modified);
					fileChangedEvent(key, FileListenerEvent.FILE_EVENT_TYPE_DELETE);
				}
				else if (cur_modified.longValue() > origin_modified.longValue())
				{
					_FileMap.put(key, cur_modified);

					fileChangedEvent(key, FileListenerEvent.FILE_EVENT_TYPE_MODIFY);
				}
			}

			for (Iterator iter = removedFiles.entrySet().iterator(); iter.hasNext();)
			{
				Entry entry = (Entry) iter.next();
				_FileMap.remove(entry.getKey());
			}

			for (Iterator iter = curFiles.entrySet().iterator(); iter.hasNext();)
			{
				Entry entry = (Entry) iter.next();
				String key = (String) entry.getKey();

				_FileMap.put(key, entry.getValue());

				fileChangedEvent(key, FileListenerEvent.FILE_EVENT_TYPE_ADD);
			}
		} catch (Exception ex)
		{
			log.error(ex);
		} finally
		{
			checkActionFlag = true;
		}
	}

	private void fileChangedEvent(String filePath, String eventType)
	{
		log.info("fileChangedEvent():: File=" + filePath + " EventType=" + eventType);

		for (Iterator iter = _FileListenerList.iterator(); iter.hasNext();)
		{
			try
			{
				FileListener listener = (FileListener) iter.next();
				listener.onFileChanged(new FileListenerEvent(filePath, eventType));
			} catch (Exception ex)
			{}
		}
	}

}