package com.cim.idm.framework.util.resource;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.util.StringUtils;
import com.cim.idm.framework.util.file.CollectionUtil;
import com.cim.idm.framework.util.file.FileUtil;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.util.bundle<br>
 *  NAME    : BundleUtil.java<br>
 *  TYPE    : JAVA<br>
 *  DESCRIPTION : bundle-related utility
 *
 ****************************************************************************
 */

public class ResourceUtil
{

	private static Log			log						= LogFactory.getLog(ResourceUtil.class);

	private static final String	FILTER_NAME_INTERFACE	= "objectclass";
	private static final String	FILTER_NAME_BEAN		= "org.eclipse.gemini.blueprint.bean.name";

	public static List<org.w3c.dom.Document> getBundleBpels(String basePath, String resourcePath )
	{
		if(!basePath.startsWith(".jar"))
			basePath = basePath + ".jar";
		JarFile jar = null;
		try {
			jar = new JarFile(basePath);
		} catch (IOException e) {
			log.error("The JAR File:" + basePath + " Not found, when detect BPEL Files");
			log.error(e);
			return null;
		}
		return getBundleBpels(jar, resourcePath, new ArrayList<File>(), new ArrayList<String>());
	}
	
	
	/**
	 * fine the .bpel in jarFile and it is made in document
	 * 
	 * @param bundle
	 * @param resourcePath
	 * @param bpelAllList
	 * @param failedList
	 * @return
	 */
	public static List<org.w3c.dom.Document> getBundleBpels(JarFile jar, String resourcePath, List<File> bpelAllList, List<String> failedList )
	{
		List<org.w3c.dom.Document> docList = new ArrayList<org.w3c.dom.Document>();
		String entryName = null;
		
		try
		{
//			BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
//			String path = baseData.getBundleFile().getBaseFile().getAbsolutePath();
//			String path = getBundleBasePath(bundle);

			Enumeration<JarEntry> entries = jar.entries();
			
			while (entries.hasMoreElements())
			{
				JarEntry jarEntry = (JarEntry) entries.nextElement();
				entryName = jarEntry.getName();
				
				if (resourcePath.startsWith("/"))
				{
					if (!entryName.startsWith("/"))
						entryName = "/" + entryName;
				}
				
				if ( entryName.toLowerCase().endsWith(".bpel"))
				{
					try
					{
						InputStream inputStream = jar.getInputStream(jarEntry);
						javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
						javax.xml.parsers.DocumentBuilder builder = factory.newDocumentBuilder();
	
						docList.add(builder.parse(inputStream));
					}
					catch ( Exception ex )
					{
						failedList.add( entryName );
						log.error( ex, ex );
					}
					finally
					{
						bpelAllList.add( new File( entryName ));
					}
				}
			}
			return docList;
		} catch (Exception e)
		{
			log.error(e, e);
			return null;
		}
	}

	
	
	/**
	 * Create BufferedInputStream object for ResourceFile
	 * 
	 * @param bundle
	 * @param resourcePath
	 * @param fileDescriptor
	 * @return InputStream
	 */
	@SuppressWarnings("restriction")
	public static InputStream getBundleResourceFile(String basePath, String resourcePath, String fileDescriptor)
	{
		try
		{

//			BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
//			String path = baseData.getBundleFile().getBaseFile().getAbsolutePath();
			
//			String path = getBundleBasePath(bundle);
			String path = basePath;

			//path = path + ".jar";
			File file = new File(path + File.separator + resourcePath + "." + fileDescriptor);
			if (file.exists())
			{
				return new BufferedInputStream(new FileInputStream(file.getPath()));
			}

			if(!basePath.endsWith(".jar"))
				path = path + ".jar";
			JarFile jar = new JarFile(path);

			Enumeration<JarEntry> entries = jar.entries();
			while (entries.hasMoreElements())
			{
				JarEntry jarEntry = (JarEntry) entries.nextElement();
				String name = jarEntry.getName();
				if (resourcePath.startsWith("/"))
				{
					if (!name.startsWith("/"))
						name = "/" + name;
				}
				if (name.startsWith(resourcePath) && name.toLowerCase().endsWith("." + fileDescriptor))
				{
					return jar.getInputStream(jarEntry);
				}
			}
			return null;
		} catch (Exception e)
		{
			log.error(e, e);
			return null;
		}
	}

	/**
	 * @param bundle
	 * @param resourcePath
	 * @return
	 */
	@SuppressWarnings("restriction")
	public static InputStream getBundleResourceInputStream(String basePath, String resourcePath)
	{
		
		try
		{
//			BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
//			String path = baseData.getBundleFile().getBaseFile().getAbsolutePath();
			
//			String path = getBundleBasePath(bundle);
			String path = basePath;
			//path = path + ".jar";
			JarFile jar = new JarFile(path);
			Enumeration<JarEntry> entries = jar.entries();
			while (entries.hasMoreElements())
			{
				JarEntry jarEntry = (JarEntry) entries.nextElement();
				String name = jarEntry.getName();
				if (resourcePath.startsWith("/"))
					name = "/" + name;
				if (name.equals(resourcePath))
				{
					InputStream inputStream = jar.getInputStream(jarEntry);
					return inputStream;
				}
			}
		} catch (Exception e)
		{
			log.error(e, e);
		}
		return null;
	}

	/**
	 * File creation with bundle resources received
	 * 
	 * @param bundle
	 * @param resourcePath
	 * @return
	 */
	public static File getBundleResource(String basePath, String resourcePath)
	{
		try
		{
			File file = new File(resourcePath);
			if (file.exists())
				return file;
			
//			String path = getBundleBasePath(bundle);
			String path = basePath;
			file = new File(path + File.separator + resourcePath);
			if (!file.exists())
			{
				log.debug("Could not load bundle resource : " + path);
				return null;
			}
			return file;
		} catch (IllegalArgumentException e)
		{
			// TODO Auto-generated catch block
			log.error(e, e);
			return null;
		} catch (Exception e)
		{
			log.error(e, e);
			return null;

		}
	}

	/**
	 * File creation with bundle resources received
	 * 
	 * @param bundle
	 * @param fileName
	 * @return
	 */
	public static File getBundleResourceFile(String basePath, String fileName)
	{
		try
		{
			File file = new File(fileName);
			if (file.exists())
				return file;

//			BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
//			String path = baseData.getBundleFile().getBaseFile().getAbsolutePath();
//			String path = getBundleBasePath(bundle);
			String path = basePath;
			file = new File(path + File.separator + fileName);
			if (!file.exists())
			{
				log.debug("Could not load bundle resource : " + path);
				return null;
			}
			return file;
		} catch (IllegalArgumentException e)
		{
			// TODO Auto-generated catch block
			log.error(e, e);
			return null;
		}
	}


	
	
	/**
	 * @param bundle
	 * @param resourcePath
	 * @param filter
	 * @param addResourcePath
	 * @return
	 * @throws Exception
	 */
	public static String[] getBundleResource(String basePath, String resourcePath, String filter, boolean addResourcePath)
			throws Exception
	{
//		if (!isExistBundleResource(bundle, resourcePath))
//			return null;

		File file = getBundleResource(basePath, resourcePath);
		//bpelFile = null;
		if (file != null)
		{
			List<File> fileList = FileUtil.getFileList(file, filter, false);
			String[] files = new String[fileList.size()];
			for (int i = 0; i < fileList.size(); i++)
			{
				if (resourcePath.endsWith("/"))
					files[i] = resourcePath + ((File) fileList.get(i)).getName();
				else
					files[i] = resourcePath + "/" + ((File) fileList.get(i)).getName();
			}
			return files;
		}
		else
		{
			List<String> files = new ArrayList<String>();
			/** 
			 *  when framework(org.eclipse.osgi_xxx.jar) version earlier 3.10 <br/>
			 *  for example : org.eclipse.osgi-3.9.1.v20130814-1242.jar 
			 */
//			BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
//			String path = baseData.getBundleFile().getBaseFile().getAbsolutePath();
			String path = basePath;
			//path = path + ".jar";
			JarFile jar = new JarFile(path);
			Enumeration<JarEntry> entries = jar.entries();
			while (entries.hasMoreElements())
			{
				JarEntry jarEntry = (JarEntry) entries.nextElement();
				String name = jarEntry.getName();
				if (name.startsWith(resourcePath) && name.endsWith(filter))
				{
					name = StringUtils.replace(name, resourcePath, "");
					if (name.length() > 0)
					{
						if (name.indexOf("/") == 0)
							name = name.substring(1, name.length());
						if (name.indexOf("/") < 0)
						{
							if (resourcePath.endsWith("/"))
								files.add(resourcePath + name);
							else
								files.add(resourcePath + "/" + name);
						}
					}
				}
			}
			String[] strFiles = new String[files.size()];
			for (int i = 0; i < strFiles.length; i++)
			{
				strFiles[i] = files.get(i);
			}

			return strFiles;
		}
	}

	
	public static String getResourceBashPath() {
		return null;
		
	}

	public static String[] getBundleResource2(String basePath, String resourcePath, String filter, boolean addResourcePath)
	{
		try
		{
//			BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
//			String path = baseData.getBundleFile().getBaseFile().getAbsolutePath();
//			String path = getBundleBasePath(bundle);
			String path = basePath;
			String resourceDir = path + File.separator + resourcePath;
			List<String> fileList = FileUtil.getFileNameList(basePath, resourceDir, ".XML", false);

			if (addResourcePath)
			{
				String newValue = "";
				for (int i = 0; i < fileList.size(); i++)
				{
					if (!resourcePath.endsWith(File.separator))
						newValue = resourcePath + File.separator + (String) fileList.get(i);
					else
						newValue = resourcePath + (String) fileList.get(i);

					fileList.set(i, newValue);
				}
			}
			return CollectionUtil.toStringArray(fileList);
		} catch (IllegalArgumentException e)
		{
			log.error(e, e);
		} catch (Exception e)
		{
			log.error(e, e);
		}
		return null;
	}

	/**
	 * Bundle resource exists or not
	 * 
	 * @param bundle
	 * @param resourcePath
	 * @return
	 */
//	public static boolean isExistResource(String basePath, String resourcePath)
//	{
//		try
//		{
//			URL url = bundle.getResource(resourcePath);
//			if (url != null)
//				return true;
//			return false;
//		} catch (Exception e)
//		{
//			log.error(e, e);
//		}
//		return false;
//	}
//	
	


}
