package com.cim.idm.framework.util.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;

import com.cim.idm.framework.IDMFrameServiceProxy;

/**
 ****************************************************************************
 * PACKAGE : com.cim.idm.framework.util.bundle<br>
 * NAME : BundleUtil.java<br>
 * TYPE : JAVA<br>
 * DESCRIPTION : bundle-related utility
 *
 ****************************************************************************
 */

public class ServiceUtil {

	private static Log log = LogFactory.getLog(ServiceUtil.class);

	private static long serviceLookupTimeout = 30000;

//	@Autowired
//	private static Environment environment;

	static {
		String timeout = System.getProperty(IDMFrameProperties.SERVICE_LOOKUP_TIMEOUT);

		if (org.apache.commons.lang3.StringUtils.isNotEmpty(timeout))
			serviceLookupTimeout = Long.parseLong(timeout);
	}

	public static long getServiceLookupTimeout() {
		return serviceLookupTimeout;
	}


	
	public static <T> List<T> getServices(ApplicationContext applicationContext, Class<T> interfaceName) {
		List<T> serviceList = new ArrayList<T>();
		Map<String, ? extends T> serviceMap = (Map<String, ? extends T>) applicationContext
				.getBeansOfType(interfaceName);
		/*
		for (Entry<String, ? extends Object> entry : serviceMap.entrySet()) {
			serviceList.add(entry.getValue());
			break;
		}
		*/
		if(serviceMap != null)
			serviceList.addAll(serviceMap.values());

		return serviceList;
	}

	@SuppressWarnings("unchecked")
	public static Object getServiceByClassName(ApplicationContext applicationContext, String className) throws ClassNotFoundException {
		Class clazz;
		try {
			clazz = Class.forName(className);
		} catch (ClassNotFoundException e) {
			log.error(e, e);
			throw e;
		}
		return applicationContext.getBean(clazz);
	}
	
	/**
	 * format first Charactor of a clazz SimpleName tolowcase</br>
	 * <pre>    
	 * Examples: 
	 *             formatSimpleClassName("Hello") returns "hello"
	 * </pre>
	 * @param clazz
	 * @return 
	 */
	public static String formatSimpleClassName(Class clazz){
		String firstLowChar=clazz.getSimpleName().substring(0, 1).toLowerCase();
		String postStr = clazz.getSimpleName().substring(1);
		return firstLowChar + postStr;
	}
}
