package com.cim.idm.framework.util.time;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.util.time<br>
 *  NAME    : TimeStampUtil.java<br>
 *  TYPE    : JAVA<br>
 *  DESCRIPTION : timeStamp-related utility<br>
 *
 ****************************************************************************
 */

public class TimeStampUtil
{
	public static final String	FORMAT_TIMEKEY			= "yyyyMMddHHmmssSSSSSS";

	public static final String	FORMAT_DETAIL			= "yyyy-MM-dd HH:mm:ss.SSS";
	public static final String	FORMAT_DEFAULT			= "yyyy-MM-dd HH:mm:ss";
	public static final String	FORMAT_DAY				= "yyyy-MM-dd";

	public static final String	FORMAT_SIMPLE_DETAIL	= "yyyyMMddHHmmssSSS";
	public static final String	FORMAT_SIMPLE_DEFAULT	= "yyyyMMddHHmmss";
	public static final String	FORMAT_SIMPLE_DAY		= "yyyyMMdd";

	private static Log			log						= LogFactory.getLog(TimeStampUtil.class);

	public static String getCurrentEventTimeKey()
	{
		SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_TIMEKEY);
		return sdf.format(System.currentTimeMillis());
	}

	public static String getEventTimeKeyFromTimestamp(Timestamp timestamp)
	{
		SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_TIMEKEY);
		return sdf.format(timestamp);
	}

	public static String getCurrentTime()
	{
		long t = System.currentTimeMillis();
		Timestamp st = new Timestamp(t);
		return st.toString();
	}

	public static String getCurrentTime(String format)
	{
		long t = System.currentTimeMillis();
		Timestamp date = new Timestamp(t);

		SimpleDateFormat sdf = new SimpleDateFormat(format);
		return sdf.format(date);
	}

	public static Timestamp getCurrentTimestamp()
	{
		long t = System.currentTimeMillis();
		return new Timestamp(t);
	}

	public static String toTimeString(Timestamp timestamp)
	{
		return toTimeString(timestamp, FORMAT_DEFAULT);
	}

	/**
	 *  Create timestamp according to the format
	 * 
	 * @param timestamp
	 * @param format
	 * @return
	 */
	public static String toTimeString(Timestamp timestamp, String format)
	{
		if (timestamp == null)
			timestamp = getCurrentTimestamp();

		if (StringUtils.isEmpty(format))
			format = FORMAT_DEFAULT;

		SimpleDateFormat sdf = new SimpleDateFormat(format);
		return sdf.format(timestamp);
	}

	/**
	 * Create format according to the time
	 * 
	 * @param time
	 * @return
	 */
	public static Timestamp getTimestamp(String time)
	{
		try
		{
			SimpleDateFormat sdf = null;

			if (time.length() > 20)
				sdf = new SimpleDateFormat(FORMAT_DETAIL); // yyyy-MM-dd HH:mm:ss.SSS
			else if (time.length() == 20)
				sdf = new SimpleDateFormat(FORMAT_TIMEKEY); // yyyyMMddHHmmssSSSSSS
			else if (time.length() == 19)
				sdf = new SimpleDateFormat(FORMAT_DEFAULT); // yyyy-MM-dd HH:mm:ss
			else if (time.length() == 17)
				sdf = new SimpleDateFormat(FORMAT_SIMPLE_DETAIL); // yyyyMMddHHmmssSSS
			else if (time.length() == 14)
				sdf = new SimpleDateFormat(FORMAT_SIMPLE_DEFAULT); // yyyyMMddHHmmss
			else if (time.length() == 10)
				sdf = new SimpleDateFormat(FORMAT_DAY); // yyyy-MM-dd
			else if (time.length() == 8)
				sdf = new SimpleDateFormat(FORMAT_SIMPLE_DAY); // yyyyMMdd
			else
			{
				log.warn("could not create Timestamp : " + time);
				return null;
			}
			Date date = sdf.parse(time);
			return new Timestamp(date.getTime());
		} catch (Exception e)
		{
			return null;
		}
	}
}
