package com.cim.idm.framework.util.time;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;

import com.cim.idm.framework.IDMFrameServiceProxy;
import com.cim.idm.framework.data.EventInfo;
import com.cim.idm.framework.exception.IDMFrameDBErrorSignal;
import com.cim.idm.framework.orm.SqlTemplate;

/**
 ****************************************************************************
 *  PACKAGE : com.cim.idm.framework.util.time<br>
 *  NAME    : TimeUtils.java<br>
 *  TYPE    : JAVA<br>
 *  DESCRIPTION : time-related utility<br>
 *
 ****************************************************************************
 */

public class TimeUtils extends TimeStampUtil implements InitializingBean
{
	public static final String	FORMAT_SEQTIMEKEY	= "yyyyMMddHHmmssSSS";

	private static Log			log					= LogFactory.getLog(TimeUtils.class);
	private static boolean		useSequence4TimeKey	= false;
	private static boolean		useDB4TimeKey		= false;

	private static SqlTemplate sqlTemplate;
	
	public static boolean isUseSequence4TimeKey()
	{
		return useSequence4TimeKey;
	}

	public static void setUseSequence4TimeKey(boolean useSequence4TimeKey)
	{
		TimeUtils.useSequence4TimeKey = useSequence4TimeKey;
	}

	public static boolean isUseDB4TimeKey()
	{
		return useDB4TimeKey;
	}

	public static void setUseDB4TimeKey(boolean useDB4TimeKey)
	{
		TimeUtils.useDB4TimeKey = useDB4TimeKey;
	}

	public SqlTemplate getSqlTemplate() {
		return sqlTemplate;
	}

	public void setSqlTemplate(SqlTemplate sqlTemplate) {
		TimeUtils.sqlTemplate = sqlTemplate;
	}
	
	@Override
	public void afterPropertiesSet() throws Exception
	{
		try
		{
			String useSequenceTimekey = System.getProperty("idmframework.sequencetimekey", "true");
			useSequence4TimeKey = Boolean.parseBoolean(useSequenceTimekey);
			log.info("Use sequence for timekey : " + useSequence4TimeKey +" ,Sequence is supplied only by Oracle ");
			
			String useDBTimekey = System.getProperty("idmframework.dbtimekey", "false");
			useDB4TimeKey = Boolean.parseBoolean(useDBTimekey);
			log.info("Use db systemtime for timekey : " + useDB4TimeKey);
			//set sqlTemplate
			setSqlTemplate(IDMFrameServiceProxy.getSqlTemplate());
		} catch (Exception ex)
		{
			log.warn(ex);
		}
	}
	
	/**
	 * get current time stamp 
	 */
	public static Timestamp getCurrentTimestamp4Timekey()
	{
		if (useDB4TimeKey)
		{
			Timestamp currTime = null;
			String sql = "";
			if("Oracle".equalsIgnoreCase(SqlTemplate.RDB_TYPE)) {
				sql = "SELECT SYSTIMESTAMP FROM DUAL ";
			}else if("MySQL".equalsIgnoreCase(SqlTemplate.RDB_TYPE)) {
				sql = "SELECT CURRENT_TIMESTAMP() ";
			}else if("Microsoft SQL Server".equalsIgnoreCase(SqlTemplate.RDB_TYPE)) {
				sql = "SELECT getDate() ";
			}else if("PostgreSQL".equalsIgnoreCase(SqlTemplate.RDB_TYPE)) {
				sql = "SELECT current_timestamp ";
			}
			
			currTime = (Timestamp) sqlTemplate.queryForObject(sql, Timestamp.class, new Object[0]);
			if ( log.isDebugEnabled() ) log.debug("Current DB Timestamp is " + currTime);
			return currTime;
		}
		else
		{
			Timestamp currTime = new Timestamp(System.currentTimeMillis());
			if ( log.isDebugEnabled() ) log.debug("Current System Timestamp is " + currTime);
			return currTime;
		}
	}	

	/**
	 * Change current time according to the format and key generation
	 * @throws Exception 
	 */
	public static String getCurrentEventTimeKey()
	{
		String currentTimeKey;
		if (useSequence4TimeKey)
		{
			SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_SEQTIMEKEY);
			currentTimeKey = sdf.format(getCurrentTimestamp4Timekey());
			int nextVal = getNextTimeSequence();
			currentTimeKey = currentTimeKey + String.format("%03d", nextVal);
		}
		else
		{
			SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_TIMEKEY);
			currentTimeKey = sdf.format(getCurrentTimestamp4Timekey());
		}

		return currentTimeKey;

		//		String sql;
		//		if (useSequence4TimeKey)
		//		{
		//			sql = "select concat(to_char(systimestamp, 'YYYYMMDDHH24MISSFF3'), lpad(TIMEKEYID.nextval, 3, '0') ) as timekey from dual";
		//		}
		//		else
		//		{
		//			sql = "select to_char(systimestamp, 'YYYYMMDDHH24MISSFF6') as timekey from dual";
		//		}
		//
		//		List<Map<String, Object>> result = GenericServiceProxy.getSqlMesTemplate().queryForList(sql);
		//		String currentTimeKey = (String)result.get(0).get("TIMEKEY");
		//
		//		return currentTimeKey;
	}

	/**
	 * Change the received timestamp according to the format and key generation
	 * 
	 * @param timestamp
	 * @return
	 * @throws Exception 
	 */
	public static String getEventTimeKeyFromTimestamp(Timestamp timestamp)
	{
		//		SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_TIMEKEY);
		//		return sdf.format(timestamp);
		String currentTimeKey;
		if (useSequence4TimeKey)
		{
			SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_SEQTIMEKEY);
			currentTimeKey = sdf.format(timestamp);

			int nextVal = getNextTimeSequence();
			currentTimeKey = currentTimeKey + String.format("%03d", nextVal);
		}
		else
		{
			SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_TIMEKEY);
			currentTimeKey = sdf.format(timestamp);
		}

		return currentTimeKey;
	}

	public static int getNextTimeSequence()
	{
		String sql = "";
		int nextVal = 0;
		if("Oracle".equalsIgnoreCase(sqlTemplate.getRDBType())) {
			sql = "select TIMEKEYID.nextval from dual";
			nextVal = IDMFrameServiceProxy.getSqlTemplate().queryForInt(sql);
		
		}else if("PostgreSQL".equalsIgnoreCase(sqlTemplate.getRDBType())) {
			sql = "select TIMEKEYID.nextval";
			nextVal = IDMFrameServiceProxy.getSqlTemplate().queryForInt(sql);
		}
		else {
			throw new IDMFrameDBErrorSignal("SequenceError","Sequence is supplied by Oracle or PostgreSQL");
		}
		return nextVal;
	}

	/**
	 * Cut only realTime in received timekey, Create the key by changing to fit the format it
	 * 
	 * @param timeKey
	 * @return
	 */
	public static Timestamp getTimestampByTimeKey(String timeKey)
	{
		try
		{
			if (useSequence4TimeKey)
			{
				String realTime = timeKey.substring(0, 17);
				SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_SEQTIMEKEY);
				Date date = sdf.parse(realTime);
				return new Timestamp(date.getTime());
			}
			else
			{
				SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_TIMEKEY);
				Date date = sdf.parse(timeKey);
				return new Timestamp(date.getTime());
			}
		} catch (Throwable ex)
		{
			return null;
		}
	}

	/**
	 * generate eventTimekey and set to EventInfo
	 * 
	 * @param lastEventtimeKey
	 *            : DataInfo Object in lastEventtimeKey value
	 * @param eventInfo
	 *            if eventTimekey and eventTime does not exist in Eventinfo, set their values.
	 * @throws Exception 
	 */
	public static void setTime4EventInfo(String lastEventtimeKey, EventInfo eventInfo)
	{
		Timestamp currentTimeStamp = null;
		
		if (StringUtils.isEmpty(eventInfo.getEventTimeKey()))
		{
			currentTimeStamp = getCurrentTimestamp4Timekey();
			
			String eventTimeKey = TimeUtils.getEventTimeKeyFromTimestamp(currentTimeStamp);
			/*
			 *  set sequence to be create for cycle. When two or more events occur at the same time, 
			 *  for the time sequence  change from 999 to 0, the logic that recreate the timekey..
			 */
			try
			{
				if (TimeUtils.isUseSequence4TimeKey()
					&& StringUtils.isNotEmpty(lastEventtimeKey)
					&& lastEventtimeKey.compareTo(eventTimeKey) >= 0)
				{
					String lastTime;
					String lastSeq;
					if (lastEventtimeKey.length() >= 17)
					{
						lastTime = lastEventtimeKey.substring(0, 17);
						lastSeq = lastEventtimeKey.substring(17);
					}
					else
					{
						lastTime = lastEventtimeKey;
						lastSeq = "000";
					}
					lastSeq = String.format("%03d", new Integer(lastSeq));

					String eventTime = eventTimeKey.substring(0, 17);
					String eventSeq = eventTimeKey.substring(17);
					if (eventTime.equals(lastTime) && lastSeq.compareTo(eventSeq) >= 0)
					{
						// in case of big time difference, it can look looping, so retry just once.
						currentTimeStamp = getCurrentTimestamp4Timekey();
						eventTimeKey = TimeUtils.getEventTimeKeyFromTimestamp(currentTimeStamp);
						//						setTime4EventInfo(lastEventtimeKey, eventInfo);  
						//						return;
					}
				}
			} 
			catch (Throwable e)
			{
				if (log.isDebugEnabled())
					log.debug(e);
			}

			eventInfo.setEventTimeKey(eventTimeKey);
			
			if ( log.isDebugEnabled() ) log.debug("Timekey is assigned : " + eventInfo.getEventTimeKey());
		}
		else
		{
			if ( log.isDebugEnabled() ) log.debug("Timekey is already assigned : " + eventInfo.getEventTimeKey());
		}

		if (eventInfo.getEventTime() == null)
		{
			if ( currentTimeStamp == null ) currentTimeStamp = getCurrentTimestamp4Timekey();
			eventInfo.setEventTime(currentTimeStamp);
		}
	}

	/**
	 * Create format according to the time
	 * 
	 * @param time
	 * @return
	 */
	public static Timestamp getTimestamp(String time)
	{
		try
		{
			SimpleDateFormat sdf = null;
			
			if (time.length() > 20)
				sdf = new SimpleDateFormat(FORMAT_DETAIL); // yyyy-MM-dd HH:mm:ss.SSS
			else if (time.length() == 20)// yyyyMMddHHmmssSSSSSS
			{
				return getTimestampByTimeKey(time);
			}
			else if (time.length() == 19)
				sdf = new SimpleDateFormat(FORMAT_DEFAULT); // yyyy-MM-dd HH:mm:ss
			else if (time.length() == 17)
				sdf = new SimpleDateFormat(FORMAT_SIMPLE_DETAIL); // yyyyMMddHHmmssSSS
			else if (time.length() == 14)
				sdf = new SimpleDateFormat(FORMAT_SIMPLE_DEFAULT); // yyyyMMddHHmmss
			else if (time.length() == 10)
				sdf = new SimpleDateFormat(FORMAT_DAY); // yyyy-MM-dd
			else if (time.length() == 8)
				sdf = new SimpleDateFormat(FORMAT_SIMPLE_DAY); // yyyyMMdd
			else
			{
				log.warn("could not create Timestamp : " + time);
				return null;
			}
			Date date = sdf.parse(time);
			return new Timestamp(date.getTime());
		} catch (Exception e)
		{
			return null;
		}
	}

}
